@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 10547 2010-04-29 15:39:29Z charles $
+ * $Id: actions.c 11263 2010-09-24 15:22:53Z charles $
  */
 
 #include <string.h>
@@ -21,13 +21,13 @@
 #include "conf.h"
 #include "tr-core.h"
 #include "tr-prefs.h"
-#include "lock.h"
 
-#include "logo-24.h"
-#include "logo-48.h"
-#include "ratio-icon.h"
-#include "options-icon.h"
-#include "turtles.h"
+#include "icon-lock.h"
+#include "icon-logo-24.h"
+#include "icon-logo-48.h"
+#include "icon-ratio.h"
+#include "icon-turtle.h"
+#include "icon-utilities.h"
 
 #define UNUSED G_GNUC_UNUSED
 
@@ -57,10 +57,9 @@ static GtkRadioActionEntry sort_radio_entries[] =
     { "sort-by-progress",  NULL, N_( "Sort by _Progress" ),  NULL, NULL, 2 },
     { "sort-by-ratio",     NULL, N_( "Sort by Rati_o" ),     NULL, NULL, 3 },
     { "sort-by-state",     NULL, N_( "Sort by Stat_e" ),     NULL, NULL, 4 },
-    { "sort-by-tracker",   NULL, N_( "Sort by T_racker" ),   NULL, NULL, 5 },
-    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 6 },
-    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 7 },
-    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 8 }
+    { "sort-by-age",       NULL, N_( "Sort by A_ge" ),       NULL, NULL, 5 },
+    { "sort-by-time-left", NULL, N_( "Sort by Time _Left" ), NULL, NULL, 6 },
+    { "sort-by-size",      NULL, N_( "Sort by Si_ze" ),      NULL, NULL, 7 }
 };
 
 static void
@@ -120,7 +119,7 @@ static GtkActionEntry entries[] =
     { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
     { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
-    { "relocate-torrent", NULL, N_("Set _Location" ), NULL, NULL, G_CALLBACK( action_cb ) },
+    { "relocate-torrent", NULL, N_("Set _Location..." ), NULL, NULL, G_CALLBACK( action_cb ) },
     { "remove-torrent", GTK_STOCK_REMOVE, NULL, "Delete", N_( "Remove torrent" ), G_CALLBACK( action_cb ) },
     { "delete-torrent", GTK_STOCK_DELETE, N_( "_Delete Files and Remove" ), "<shift>Delete", NULL, G_CALLBACK( action_cb ) },
     { "new-torrent", GTK_STOCK_NEW, N_( "_New..." ), NULL, N_( "Create a torrent" ), G_CALLBACK( action_cb ) },
@@ -148,7 +147,7 @@ static const BuiltinIconInfo my_fallback_icons[] =
     { tr_icon_logo_24,  TRAY_ICON            },
     { tr_icon_logo_48,  NOTIFICATION_ICON    },
     { tr_icon_lock,     "transmission-lock"  },
-    { options_icon,     "options"            },
+    { utilities_icon,   "utilities"          },
     { blue_turtle,      "alt-speed-on"       },
     { grey_turtle,      "alt-speed-off"      },
     { ratio_icon,       "ratio"              }
