@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 10634 2010-05-06 17:02:31Z charles $
+ * $Id: bencode.c 11356 2010-10-26 19:30:35Z charles $
  */
 
 #include <assert.h>
@@ -18,10 +18,16 @@
 #include <stdlib.h> /* realpath() */
 #include <string.h>
 
+#ifdef WIN32 /* tr_mkstemp() */
+ #include <fcntl.h>
+ #define _S_IREAD 256
+ #define _S_IWRITE 128
+#endif
+
 #include <sys/types.h> /* stat() */
 #include <sys/stat.h> /* stat() */
 #include <locale.h>
-#include <unistd.h> /* stat(), close() */
+#include <unistd.h> /* stat() */
 
 #include <event.h> /* struct evbuffer */
 
@@ -29,6 +35,7 @@
 
 #include "transmission.h"
 #include "bencode.h"
+#include "fdlimit.h" /* tr_close_file() */
 #include "json.h"
 #include "list.h"
 #include "platform.h" /* TR_PATH_MAX */
@@ -59,8 +66,7 @@ isSomething( const tr_benc * val )
 }
 
 static void
-tr_bencInit( tr_benc * val,
-             int       type )
+tr_bencInit( tr_benc * val, char type )
 {
     memset( val, 0, sizeof( *val ) );
     val->type = type;
@@ -423,6 +429,19 @@ tr_bencListChild( tr_benc * val,
     return ret;
 }
 
+int
+tr_bencListRemove( tr_benc * list, size_t i )
+{
+    if( tr_bencIsList( list ) && ( i < list->val.l.count ) )
+    {
+        tr_bencFree( &list->val.l.vals[i] );
+        tr_removeElementFromArray( list->val.l.vals, i, sizeof( tr_benc ), list->val.l.count-- );
+        return 1;
+    }
+
+    return 0;
+}
+
 static void
 tr_benc_warning( const char * err )
 {
@@ -449,10 +468,9 @@ tr_bencGetInt( const tr_benc * val,
 }
 
 tr_bool
-tr_bencGetStr( const tr_benc * val,
-               const char **   setme )
+tr_bencGetStr( const tr_benc * val, const char ** setme )
 {
-    const int success = tr_bencIsString( val );
+    const tr_bool success = tr_bencIsString( val );
 
     if( success )
         *setme = getStr( val );
@@ -923,18 +941,16 @@ struct SaveNode
     int *            children;
 };
 
-static struct SaveNode*
-nodeNewDict( const tr_benc * val )
+static void
+nodeInitDict( struct SaveNode * node, const tr_benc * val )
 {
     int               i, j;
     int               nKeys;
-    struct SaveNode * node;
     struct KeyIndex * indices;
 
     assert( tr_bencIsDict( val ) );
 
     nKeys = val->val.l.count / 2;
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
     node->children = tr_new0( int, nKeys * 2 );
 
@@ -955,53 +971,40 @@ nodeNewDict( const tr_benc * val )
 
     assert( node->childCount == nKeys * 2 );
     tr_free( indices );
-    return node;
 }
 
-static struct SaveNode*
-nodeNewList( const tr_benc * val )
+static void
+nodeInitList( struct SaveNode * node, const tr_benc * val )
 {
     int               i, n;
-    struct SaveNode * node;
 
     assert( tr_bencIsList( val ) );
 
     n = val->val.l.count;
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
     node->childCount = n;
     node->children = tr_new0( int, n );
     for( i = 0; i < n; ++i ) /* a list's children don't need to be reordered */
         node->children[i] = i;
-
-    return node;
 }
 
-static struct SaveNode*
-nodeNewLeaf( const tr_benc * val )
+static void
+nodeInitLeaf( struct SaveNode * node, const tr_benc * val )
 {
-    struct SaveNode * node;
-
     assert( !isContainer( val ) );
 
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
-    return node;
 }
 
-static struct SaveNode*
-nodeNew( const tr_benc * val )
+static void
+nodeInit( struct SaveNode * node, const tr_benc * val )
 {
-    struct SaveNode * node;
+    static const struct SaveNode INIT_NODE = { NULL, 0, 0, 0, NULL };
+    *node = INIT_NODE;
 
-    if( tr_bencIsList( val ) )
-        node = nodeNewList( val );
-    else if( tr_bencIsDict( val ) )
-        node = nodeNewDict( val );
-    else
-        node = nodeNewLeaf( val );
-
-    return node;
+         if( tr_bencIsList( val ) ) nodeInitList( node, val );
+    else if( tr_bencIsDict( val ) ) nodeInitDict( node, val );
+    else                            nodeInitLeaf( node, val );
 }
 
 typedef void ( *BencWalkFunc )( const tr_benc * val, void * user_data );
@@ -1027,13 +1030,15 @@ bencWalk( const tr_benc          * top,
           const struct WalkFuncs * walkFuncs,
           void                   * user_data )
 {
-    tr_ptrArray stack = TR_PTR_ARRAY_INIT;
+    int stackSize = 0;
+    int stackAlloc = 64;
+    struct SaveNode * stack = tr_new( struct SaveNode, stackAlloc );
 
-    tr_ptrArrayAppend( &stack, nodeNew( top ) );
+    nodeInit( &stack[stackSize++], top );
 
-    while( !tr_ptrArrayEmpty( &stack ) )
+    while( stackSize > 0 )
     {
-        struct SaveNode * node = tr_ptrArrayBack( &stack );
+        struct SaveNode * node = &stack[stackSize-1];
         const tr_benc *   val;
 
         if( !node->valIsVisited )
@@ -1050,9 +1055,8 @@ bencWalk( const tr_benc          * top,
         {
             if( isContainer( node->val ) )
                 walkFuncs->containerEndFunc( node->val, user_data );
-            tr_ptrArrayPop( &stack );
+            --stackSize;
             tr_free( node->children );
-            tr_free( node );
             continue;
         }
 
@@ -1075,17 +1079,27 @@ bencWalk( const tr_benc          * top,
                     break;
 
                 case TR_TYPE_LIST:
-                    if( val != node->val )
-                        tr_ptrArrayAppend( &stack, nodeNew( val ) );
-                    else
+                    if( val == node->val )
                         walkFuncs->listBeginFunc( val, user_data );
+                    else {
+                        if( stackAlloc == stackSize ) {
+                            stackAlloc *= 2;
+                            stack = tr_renew( struct SaveNode, stack, stackAlloc );
+                        }
+                        nodeInit( &stack[stackSize++], val );
+                    }
                     break;
 
                 case TR_TYPE_DICT:
-                    if( val != node->val )
-                        tr_ptrArrayAppend( &stack, nodeNew( val ) );
-                    else
+                    if( val == node->val )
                         walkFuncs->dictBeginFunc( val, user_data );
+                    else {
+                        if( stackAlloc == stackSize ) {
+                            stackAlloc *= 2;
+                            stack = tr_renew( struct SaveNode, stack, stackAlloc );
+                        }
+                        nodeInit( &stack[stackSize++], val );
+                    }
                     break;
 
                 default:
@@ -1095,7 +1109,7 @@ bencWalk( const tr_benc          * top,
             }
     }
 
-    tr_ptrArrayDestruct( &stack, NULL );
+    tr_free( stack );
 }
 
 /****
@@ -1173,42 +1187,35 @@ static const struct WalkFuncs saveFuncs = { saveIntFunc,
 ***/
 
 static void
-freeDummyFunc( const tr_benc * val UNUSED,
-               void * buf          UNUSED  )
+freeDummyFunc( const tr_benc * val UNUSED, void * buf UNUSED  )
 {}
 
 static void
-freeStringFunc( const tr_benc * val,
-                void *          freeme )
+freeStringFunc( const tr_benc * val, void * unused UNUSED )
 {
     if( stringIsAlloced( val ) )
-        tr_ptrArrayAppend( freeme, val->val.s.str.ptr );
+        tr_free( val->val.s.str.ptr );
 }
 
 static void
-freeContainerBeginFunc( const tr_benc * val,
-                        void *          freeme )
+freeContainerEndFunc( const tr_benc * val, void * unused UNUSED )
 {
-    tr_ptrArrayAppend( freeme, val->val.l.vals );
+    tr_free( val->val.l.vals );
 }
 
 static const struct WalkFuncs freeWalkFuncs = { freeDummyFunc,
                                                 freeDummyFunc,
                                                 freeDummyFunc,
                                                 freeStringFunc,
-                                                freeContainerBeginFunc,
-                                                freeContainerBeginFunc,
-                                                freeDummyFunc };
+                                                freeDummyFunc,
+                                                freeDummyFunc,
+                                                freeContainerEndFunc };
 
 void
 tr_bencFree( tr_benc * val )
 {
     if( isSomething( val ) )
-    {
-        tr_ptrArray a = TR_PTR_ARRAY_INIT;
-        bencWalk( val, &freeWalkFuncs, &a );
-        tr_ptrArrayDestruct( &a, tr_free );
-    }
+        bencWalk( val, &freeWalkFuncs, NULL );
 }
 
 /***
@@ -1332,8 +1339,8 @@ jsonRealFunc( const tr_benc * val, void * vdata )
     struct jsonWalk * data = vdata;
     char locale[128];
 
-    if( fabs( val->val.d ) < 0.00001 )
-        evbuffer_add( data->out, "0", 1 );
+    if( fabs( val->val.d - (int)val->val.d ) < 0.00001 )
+        evbuffer_add_printf( data->out, "%d", (int)val->val.d );
     else {
         /* json requires a '.' decimal point regardless of locale */
         tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
@@ -1615,6 +1622,31 @@ tr_bencToStr( const tr_benc * top, tr_fmt_mode mode, int * len )
     return ret;
 }
 
+/* portability wrapper for mkstemp(). */
+static int
+tr_mkstemp( char * template )
+{
+#ifdef WIN32
+    const int flags = O_RDWR | O_BINARY | O_CREAT | O_EXCL | _O_SHORT_LIVED;
+    const mode_t mode = _S_IREAD | _S_IWRITE;
+    mktemp( template );
+    return open( template, flags, mode );
+#else
+    return mkstemp( template );
+#endif
+}
+
+/* portability wrapper for fsync(). */
+static void
+tr_fsync( int fd )
+{
+#ifdef WIN32
+    _commit( fd );
+#else
+    fsync( fd );
+#endif
+}
+
 int
 tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
 {
@@ -1624,13 +1656,13 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
     char buf[TR_PATH_MAX];
 
     /* follow symlinks to find the "real" file, to make sure the temporary
-     * we build with mkstemp() is created on the right partition */
-    if( realpath( filename, buf ) != NULL )
+     * we build with tr_mkstemp() is created on the right partition */
+    if( tr_realpath( filename, buf ) != NULL )
         filename = buf;
 
     /* if the file already exists, try to move it out of the way & keep it as a backup */
     tmp = tr_strdup_printf( "%s.tmp.XXXXXX", filename );
-    fd = mkstemp( tmp );
+    fd = tr_mkstemp( tmp );
     if( fd >= 0 )
     {
         int len;
@@ -1639,10 +1671,13 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
 
         if( write( fd, str, len ) == (ssize_t)len )
         {
-            fsync( fd );
-            close( fd );
+            struct stat sb;
+            const tr_bool already_exists = !stat( filename, &sb ) && S_ISREG( sb.st_mode );
 
-            if( !unlink( filename ) || ( errno == ENOENT ) )
+            tr_fsync( fd );
+            tr_close_file( fd );
+
+            if( !already_exists || !unlink( filename ) )
             {
                 tr_dbg( "Renaming \"%s\" as \"%s\"", tmp, filename );
 
@@ -1668,7 +1703,7 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
         {
             err = errno;
             tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
-            close( fd );
+            tr_close_file( fd );
             unlink( tmp );
         }
 
@@ -1710,3 +1745,4 @@ tr_bencLoadFile( tr_benc * setme, tr_fmt_mode mode, const char * filename )
     tr_free( content );
     return err;
 }
+
