@@ -1,6 +1,5 @@
 /*
  * This file Copyright (C) 2009-2010 Mnemosyne LLC
-            trackers[trackerCount].id = 0;
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -8,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: metainfo.c 10575 2010-05-01 04:12:15Z charles $
+ * $Id: metainfo.c 11398 2010-11-11 15:31:11Z charles $
  */
 
 #include <assert.h>
@@ -43,7 +42,7 @@ tr_metainfoGetBasename( const tr_info * inf )
     for( pch=name; pch && *pch; ++pch )
         if( *pch == '/' )
             *pch = '_';
-   
+
     ret = tr_strdup_printf( "%s.%16.16s", name, inf->hashString );
 
     tr_free( name );
@@ -169,7 +168,7 @@ getfile( char ** setme, const char * root, tr_benc * path )
             }
         }
 
-        *setme = tr_utf8clean( (char*)EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ), NULL );
+        *setme = tr_utf8clean( (char*)EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
         /* fprintf( stderr, "[%s]\n", *setme ); */
         evbuffer_free( buf );
         success = TRUE;
@@ -360,11 +359,11 @@ geturllist( tr_info * inf,
             tr_benc * meta )
 {
     tr_benc * urls;
+    const char * url;
 
     if( tr_bencDictFindList( meta, "url-list", &urls ) )
     {
         int          i;
-        const char * url;
         const int    n = tr_bencListSize( urls );
 
         inf->webseedCount = 0;
@@ -374,6 +373,12 @@ geturllist( tr_info * inf,
             if( tr_bencGetStr( tr_bencListChild( urls, i ), &url ) )
                 inf->webseeds[inf->webseedCount++] = tr_strdup( url );
     }
+    else if( tr_bencDictFindStr( meta, "url-list", &url ) ) /* handle single items in webseeds */
+    {
+        inf->webseedCount = 1;
+        inf->webseeds = tr_new0( char*, 1 );
+        inf->webseeds[0] = tr_strdup( url );
+    }
 }
 
 static int
@@ -381,7 +386,11 @@ is_rfc2396_alnum( char ch )
 {
     return ( '0' <= ch && ch <= '9' )
         || ( 'A' <= ch && ch <= 'Z' )
-        || ( 'a' <= ch && ch <= 'z' );
+        || ( 'a' <= ch && ch <= 'z' )
+        || ch == '.'
+        || ch == '-'
+        || ch == '_'
+        || ch == '~';
 }
 
 static void
@@ -393,7 +402,7 @@ escape( char * out, const uint8_t * in, size_t in_len ) /* rfc2396 */
         if( is_rfc2396_alnum( *in ) )
             *out++ = (char) *in++;
         else
-            out += tr_snprintf( out, 4, "%%%02X", (unsigned int)*in++ );
+            out += tr_snprintf( out, 4, "%%%02x", (unsigned int)*in++ );
 
     *out = '\0';
 }
@@ -402,7 +411,6 @@ static const char*
 tr_metainfoParseImpl( const tr_session  * session,
                       tr_info           * inf,
                       tr_bool           * hasInfoDict,
-                      int               * infoDictOffset,
                       int               * infoDictLength,
                       const tr_benc     * meta_in )
 {
@@ -413,7 +421,6 @@ tr_metainfoParseImpl( const tr_session  * session,
     tr_benc *       d;
     tr_benc *       infoDict = NULL;
     tr_benc *       meta = (tr_benc *) meta_in;
-    tr_bool         err;
     tr_bool         b;
     tr_bool         isMagnet = FALSE;
 
@@ -444,6 +451,9 @@ tr_metainfoParseImpl( const tr_session  * session,
                 tr_free( inf->name );
                 inf->name = tr_strdup( str );
             }
+
+            if( !inf->name )
+                inf->name = tr_strdup( inf->hashString );
         }
         else /* not a magnet link and has no info dict... */
         {
@@ -461,18 +471,6 @@ tr_metainfoParseImpl( const tr_session  * session,
         if( infoDictLength != NULL )
             *infoDictLength = len;
 
-        if( infoDictOffset != NULL )
-        {
-            int mlen = 0;
-            char * mstr = tr_bencToStr( meta_in, TR_FMT_BENC, &mlen );
-            const char * offset = tr_memmem( mstr, mlen, bstr, len );
-            if( offset != NULL )
-                *infoDictOffset = offset - mstr;
-            tr_free( mstr );
-            if( offset == NULL )
-                return "info";
-        }
-
         tr_free( bstr );
     }
 
@@ -484,7 +482,7 @@ tr_metainfoParseImpl( const tr_session  * session,
         if( !str || !*str )
             return "name";
         tr_free( inf->name );
-        inf->name = tr_utf8clean( str, -1, &err );
+        inf->name = tr_utf8clean( str, -1 );
     }
 
     /* comment */
@@ -492,14 +490,14 @@ tr_metainfoParseImpl( const tr_session  * session,
         if( !tr_bencDictFindStr( meta, "comment", &str ) )
             str = "";
     tr_free( inf->comment );
-    inf->comment = tr_utf8clean( str, -1, &err );
+    inf->comment = tr_utf8clean( str, -1 );
 
     /* created by */
     if( !tr_bencDictFindStr( meta, "created by.utf-8", &str ) )
         if( !tr_bencDictFindStr( meta, "created by", &str ) )
             str = "";
     tr_free( inf->creator );
-    inf->creator = tr_utf8clean( str, -1, &err );
+    inf->creator = tr_utf8clean( str, -1 );
 
     /* creation date */
     if( !tr_bencDictFindInt( meta, "creation date", &i ) )
@@ -563,13 +561,11 @@ tr_metainfoParse( const tr_session * session,
                   const tr_benc    * meta_in,
                   tr_info          * inf,
                   tr_bool          * hasInfoDict,
-                  int              * infoDictOffset,
                   int              * infoDictLength )
 {
     const char * badTag = tr_metainfoParseImpl( session,
                                                 inf,
                                                 hasInfoDict,
-                                                infoDictOffset,
                                                 infoDictLength,
                                                 meta_in );
     const tr_bool success = badTag == NULL;
