@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * $Id: net.c 10570 2010-05-01 03:53:16Z charles $
+ * $Id: net.c 11311 2010-10-14 17:03:04Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -32,9 +32,12 @@
 #include <sys/types.h>
 
 #ifdef WIN32
- #include <winsock2.h> /* inet_addr */
- #include <WS2tcpip.h>
+ #define _WIN32_WINNT   0x0501
+ #include <ws2tcpip.h>
 #else
+ #include <sys/socket.h>
+ #include <netinet/in.h>
+ #include <netinet/tcp.h>
  #include <arpa/inet.h> /* inet_addr */
  #include <netdb.h>
  #include <fcntl.h>
@@ -61,8 +64,8 @@ const tr_address tr_in6addr_any = { TR_AF_INET6, { IN6ADDR_ANY_INIT } };
 const tr_address tr_inaddr_any = { TR_AF_INET, { { { { INADDR_ANY, 0x00, 0x00, 0x00 } } } } };
 
 #ifdef WIN32
-static const char *
-inet_ntop( int af, const void *src, char *dst, socklen_t cnt )
+const char *
+inet_ntop( int af, const void * src, char * dst, socklen_t cnt )
 {
     if (af == AF_INET)
     {
@@ -87,34 +90,48 @@ inet_ntop( int af, const void *src, char *dst, socklen_t cnt )
     return NULL;
 }
 
-static int
-inet_pton(int af, const char *src, void *dst)
+int
+inet_pton( int af, const char * src, void * dst )
 {
-    struct addrinfo hints;
-    struct addrinfo *res;
-    struct addrinfo *ressave;
+    struct addrinfo hints, *res, *ressave;
+    struct sockaddr_in * s4;
+    struct sockaddr_in6 * s6;
 
-    memset(&hints, 0, sizeof(struct addrinfo));
+    memset( &hints, 0, sizeof( struct addrinfo ));
     hints.ai_family = af;
+    hints.ai_flags = AI_NUMERICHOST;
 
-    if (getaddrinfo(src, NULL, &hints, &res) != 0)
-        return -1;
+    if( getaddrinfo( src, NULL, &hints, &res ) ) {
+        if( WSAGetLastError() == WSAHOST_NOT_FOUND )
+            return 0;
+        else {
+            errno = EAFNOSUPPORT;
+            return -1;
+        }
+    }
 
     ressave = res;
-
-    while (res)
-    {
-        memcpy(dst, res->ai_addr, res->ai_addrlen);
+    while( res ) {
+        switch (res->ai_family) {
+            case AF_INET:
+                s4 = (struct sockaddr_in *) res->ai_addr;
+                memcpy( dst, &s4->sin_addr, sizeof( struct in_addr ) );
+                break;
+            case AF_INET6:
+                s6 = (struct sockaddr_in6 *) res->ai_addr;
+                memcpy( dst, &s6->sin6_addr, sizeof( struct in6_addr ) );
+                break;
+            default: /* AF_UNSPEC, AF_NETBIOS */
+                break;
+        }
         res = res->ai_next;
     }
 
     freeaddrinfo(ressave);
-    return 0;
+    return 1;
 }
-
 #endif
 
-
 void
 tr_netInit( void )
 {
@@ -130,6 +147,18 @@ tr_netInit( void )
     }
 }
 
+char *
+tr_net_strerror( char * buf, size_t buflen, int err )
+{
+    *buf = '\0';
+#ifdef WIN32
+    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM, NULL, err, 0, buf, buflen, NULL );
+#else
+    tr_strlcpy( buf, tr_strerror( err ), buflen );
+#endif
+    return buf;
+}
+
 const char *
 tr_ntop( const tr_address * src, char * dst, int size )
 {
@@ -186,9 +215,6 @@ tr_compareAddresses( const tr_address * a, const tr_address * b)
 {
     static const int sizes[2] = { sizeof(struct in_addr), sizeof(struct in6_addr) };
 
-    assert( tr_isAddress( a ) );
-    assert( tr_isAddress( b ) );
-
     /* IPv6 addresses are always "greater than" IPv4 */
     if( a->type != b->type )
         return a->type == TR_AF_INET ? 1 : -1;
@@ -210,6 +236,18 @@ tr_netSetTOS( int s, int tos )
 #endif
 }
 
+int
+tr_netSetCongestionControl( int s UNUSED, const char *algorithm UNUSED )
+{
+#ifdef TCP_CONGESTION
+    return setsockopt( s, IPPROTO_TCP, TCP_CONGESTION,
+                       algorithm, strlen(algorithm) + 1 );
+#else
+    errno = ENOSYS;
+    return -1;
+#endif
+}
+
 static socklen_t
 setup_sockaddr( const tr_address        * addr,
                 tr_port                   port,
@@ -365,7 +403,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
                 fmt = _( "Couldn't bind port %d on %s: %s" );
             else
                 fmt = _( "Couldn't bind port %d on %s: %s (%s)" );
-            
+
             tr_err( fmt, port, tr_ntop_non_ts( addr ), tr_strerror( err ), hint );
         }
         tr_netCloseSocket( fd );
