@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.h 10638 2010-05-08 22:42:28Z charles $
+ * $Id: session.h 11368 2010-10-31 17:16:12Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -29,6 +29,7 @@
 
 #include "bencode.h"
 #include "bitfield.h"
+#include "utils.h"
 
 typedef enum { TR_NET_OK, TR_NET_ERROR, TR_NET_WAIT } tr_tristate_t;
 
@@ -40,12 +41,13 @@ struct tr_address;
 struct tr_announcer;
 struct tr_bandwidth;
 struct tr_bindsockets;
+struct tr_cache;
 struct tr_fdInfo;
 
 struct tr_turtle_info
 {
     /* TR_UP and TR_DOWN speed limits */
-    int speedLimit[2];
+    int speedLimit_Bps[2];
 
     /* is turtle mode on right now? */
     tr_bool isEnabled;
@@ -86,13 +88,12 @@ struct tr_session
     tr_bool                      isDHTEnabled;
     tr_bool                      isLPDEnabled;
     tr_bool                      isBlocklistEnabled;
-    tr_bool                      isProxyEnabled;
-    tr_bool                      isProxyAuthEnabled;
     tr_bool                      isTorrentDoneScriptEnabled;
     tr_bool                      isClosed;
     tr_bool                      useLazyBitfield;
     tr_bool                      isIncompleteFileNamingEnabled;
     tr_bool                      isRatioLimited;
+    tr_bool                      isIdleLimited;
     tr_bool                      isIncompleteDirEnabled;
     tr_bool                      pauseAddedTorrent;
     tr_bool                      deleteSourceTorrent;
@@ -101,7 +102,7 @@ struct tr_session
 
     int                          umask;
 
-    int                          speedLimit[2];
+    int                          speedLimit_Bps[2];
     tr_bool                      speedLimitEnabled[2];
 
     struct tr_turtle_info        turtle;
@@ -120,11 +121,20 @@ struct tr_session
 
     int                          uploadSlotsPerTorrent;
 
-    tr_port                      peerPort;
+    /* The open port on the local machine for incoming peer requests */
+    tr_port                      private_peer_port;
+
+    /**
+     * The open port on the public device for incoming peer requests.
+     * This is usually the same as private_peer_port but can differ
+     * if the public device is a router and it decides to use a different
+     * port than the one requested by Transmission.
+     */
+    tr_port                      public_peer_port;
+
     tr_port                      randomPortLow;
     tr_port                      randomPortHigh;
 
-    int                          proxyPort;
     int                          peerSocketTOS;
     char *                       peer_congestion_algorithm;
 
@@ -140,15 +150,14 @@ struct tr_session
     char *                       torrentDir;
     char *                       incompleteDir;
 
-    tr_proxy_type                proxyType;
-    char *                       proxy;
-    char *                       proxyUsername;
-    char *                       proxyPassword;
+    char *                       blocklist_url;
 
     struct tr_list *             blocklists;
     struct tr_peerMgr *          peerMgr;
     struct tr_shared *           shared;
 
+    struct tr_cache *            cache;
+
     struct tr_lock *             lock;
 
     struct tr_web *              web;
@@ -170,6 +179,8 @@ struct tr_session
     struct tr_bandwidth        * bandwidth;
 
     double                       desiredRatio;
+    
+    uint16_t                     idleLimitMinutes;
 
     struct tr_bindinfo         * public_ipv4;
     struct tr_bindinfo         * public_ipv6;
@@ -181,6 +192,12 @@ struct tr_session
     tr_bool bufferInUse;
 };
 
+static inline tr_port
+tr_sessionGetPublicPeerPort( const tr_session * session )
+{
+    return session->public_peer_port;
+}
+
 tr_bool      tr_sessionAllowsDHT( const tr_session * session );
 
 tr_bool      tr_sessionAllowsLPD( const tr_session * session );
@@ -245,4 +262,30 @@ static inline tr_bool tr_isPriority( tr_priority_t p )
         || ( p == TR_PRI_HIGH );
 }
 
+/***
+****
+***/
+
+static inline unsigned int toSpeedBytes ( unsigned int KBps ) { return KBps * tr_speed_K; }
+static inline double       toSpeedKBps  ( unsigned int Bps )  { return Bps / (double)tr_speed_K; }
+
+static inline uint64_t toMemBytes ( unsigned int MB ) { uint64_t B = tr_mem_K * tr_mem_K; B *= MB; return B; }
+static inline int      toMemMB    ( uint64_t B )      { return B / ( tr_mem_K * tr_mem_K ); }
+
+/**
+**/
+
+int  tr_sessionGetSpeedLimit_Bps( const tr_session *, tr_direction );
+int  tr_sessionGetAltSpeed_Bps  ( const tr_session *, tr_direction );
+int  tr_sessionGetRawSpeed_Bps  ( const tr_session *, tr_direction );
+int  tr_sessionGetPieceSpeed_Bps( const tr_session *, tr_direction );
+
+void tr_sessionSetSpeedLimit_Bps( tr_session *, tr_direction, int Bps );
+void tr_sessionSetAltSpeed_Bps  ( tr_session *, tr_direction, int Bps );
+
+tr_bool  tr_sessionGetActiveSpeedLimit_Bps( const tr_session  * session,
+                                            tr_direction        dir,
+                                            int               * setme );
+
+
 #endif
