@@ -25,17 +25,25 @@ THE SOFTWARE.
 #include <stdio.h>
 
 /* posix */
-#include <netinet/in.h> /* sockaddr_in */
 #include <signal.h> /* sig_atomic_t */
 #include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h> /* socket(), bind() */
 #include <unistd.h> /* close() */
-#include <fcntl.h> /* fcntl(), O_NONBLOCK */
 #include <ctype.h> /* toupper() */
+#ifdef WIN32
+  #include <w32api.h>
+  #define WINDOWS  WindowsXP  /* freeaddrinfo(),getaddrinfo(),getnameinfo() */
+  #include <inttypes.h>
+  #include <ws2tcpip.h>
+  typedef uint16_t in_port_t;			/* all missing */
+#else
+  #include <sys/types.h>
+  #include <sys/socket.h> /* socket(), bind() */
+  #include <netinet/in.h> /* sockaddr_in */
+#endif
 
 /* third party */
 #include <event.h>
+#include <evutil.h>
 
 /* libT */
 #include "transmission.h"
@@ -55,7 +63,7 @@ THE SOFTWARE.
 * This module implements the Local Peer Discovery (LPD) protocol as supported by the
 * uTorrent client application.  A typical LPD datagram is 119 bytes long.
 *
-* $Id$
+* $Id: tr-lpd.c 11357 2010-10-28 23:19:53Z charles $
 */
 
 static void event_callback( int, short, void* );
@@ -241,23 +249,6 @@ static int lpd_extractParam( const char* const str, const char* const name, int
 /**
 * @} */
 
-
-/**
-* @brief Configures additional capabilities for a socket */
-static inline int lpd_configureSocket( int sock, int add )
-{
-    /* read-modify-write socket flags */
-    int flags = fcntl( sock, F_GETFL );
-
-    if( flags < 0 )
-        return -1;
-
-    if( fcntl( sock, F_SETFL, add | flags ) == -1 )
-        return -1;
-
-    return add;
-}
-
 /**
 * @brief Initializes Local Peer Discovery for this node
 *
@@ -290,8 +281,7 @@ int tr_lpdInit( tr_session* ss, tr_address* tr_addr UNUSED )
         if( lpd_socket < 0 )
             goto fail;
 
-        /* enable non-blocking operation */
-        if( lpd_configureSocket( lpd_socket, O_NONBLOCK ) < 0 )
+        if( evutil_make_socket_nonblocking( lpd_socket ) < 0 )
             goto fail;
 
         if( setsockopt( lpd_socket, SOL_SOCKET, SO_REUSEADDR,
@@ -330,8 +320,7 @@ int tr_lpdInit( tr_session* ss, tr_address* tr_addr UNUSED )
         if( lpd_socket2 < 0 )
             goto fail;
 
-        /* enable non-blocking operation */
-        if( lpd_configureSocket( lpd_socket2, O_NONBLOCK ) < 0 )
+        if( evutil_make_socket_nonblocking( lpd_socket2 ) < 0 )
             goto fail;
 
         /* configure outbound multicast TTL */
@@ -511,7 +500,7 @@ static int tr_lpdConsiderAnnounce( tr_pex* peer, const char* const msg )
             return 0;
 
         /* determine announced peer port, refuse if value too large */
-        if( sscanf( value, "%u", &peerPort ) != 1 || peerPort > (in_port_t)-1 )
+        if( sscanf( value, "%d", &peerPort ) != 1 || peerPort > (in_port_t)-1 )
             return 0;
 
         peer->port = htons( peerPort );
