@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.h 7007 2008-11-01 15:22:32Z livings124 $
+ * $Id: PrefsController.h 11369 2010-10-31 19:05:46Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,18 +32,19 @@
     NSUserDefaults * fDefaults;
     BOOL fHasLoaded;
     
-    IBOutlet NSView * fGeneralView, * fTransfersView, * fBandwidthView, * fPeersView, * fNetworkView, * fRemoteView;
+    IBOutlet NSView * fGeneralView, * fTransfersView, * fBandwidthView, * fPeersView, * fNetworkView, * fRemoteView, * fGroupsView;
     
     NSString * fInitialString;
     
     IBOutlet NSPopUpButton * fFolderPopUp, * fIncompleteFolderPopUp, * fImportFolderPopUp;
-    IBOutlet NSTextField * fRatioStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
+    IBOutlet NSTextField * fRatioStopField, * fIdleStopField, * fQueueDownloadField, * fQueueSeedField, * fStalledField;
 
     IBOutlet NSTextField * fUploadField, * fDownloadField,
                         * fSpeedLimitUploadField, * fSpeedLimitDownloadField;
+    IBOutlet NSPopUpButton * fAutoSpeedDayTypePopUp;
     
-    IBOutlet NSTextField * fPeersGlobalField, * fPeersTorrentField, * fBlocklistMessageField, * fBlocklistDateField;
-    IBOutlet NSButton * fBlocklistEnableCheck;
+    IBOutlet NSTextField * fPeersGlobalField, * fPeersTorrentField,
+                        * fBlocklistURLField, * fBlocklistMessageField, * fBlocklistDateField;
     
     PortChecker * fPortChecker;
     IBOutlet NSTextField * fPortField, * fPortStatusField;
@@ -53,19 +54,21 @@
     NSTimer * fPortStatusTimer;
     int fPeerPort, fNatStatus;
     
-    IBOutlet NSTextField * fProxyAddressField, * fProxyPortField, * fProxyPasswordField;
-    IBOutlet NSPopUpButton * fProxyTypePopUp;
-    
     IBOutlet NSTextField * fRPCPortField, * fRPCPasswordField;
     IBOutlet NSTableView * fRPCWhitelistTable;
     NSMutableArray * fRPCWhitelistArray;
     IBOutlet NSSegmentedControl * fRPCAddRemoveControl;
+    NSString * fRPCPassword;
 }
 
-+ (void) setHandle: (tr_handle *) handle;
-+ (tr_handle *) handle;
++ (void) setHandle: (tr_session *) handle;
++ (tr_session *) handle;
+
+- (void) setAutoUpdateToBeta: (id) sender;
 
 - (void) setPort: (id) sender;
+- (void) randomPort: (id) sender;
+- (void) setRandomPortOnStart: (id) sender;
 - (void) setNat: (id) sender;
 - (void) updatePortStatus;
 - (void) portCheckerDidFinishProbing: (PortChecker *) portChecker;
@@ -77,17 +80,25 @@
 - (void) setPeersTorrent: (id) sender;
 
 - (void) setPEX: (id) sender;
+- (void) setDHT: (id) sender;
+- (void) setLPD: (id) sender;
 
 - (void) setEncryptionMode: (id) sender;
 
 - (void) setBlocklistEnabled: (id) sender;
+- (void) setBlocklistURL: (id) sender;
 - (void) updateBlocklist: (id) sender;
 - (void) setBlocklistAutoUpdate: (id) sender;
 - (void) updateBlocklistFields;
 
+- (void) setAutoStartDownloads: (id) sender;
+
 - (void) setBadge: (id) sender;
+
 - (void) resetWarnings: (id) sender;
 
+- (void) setDefaultForMagnets: (id) sender;
+
 - (void) setQueue: (id) sender;
 - (void) setQueueNumber: (id) sender;
 
@@ -97,34 +108,35 @@
 - (void) setDownloadLocation: (id) sender;
 - (void) folderSheetShow: (id) sender;
 - (void) incompleteFolderSheetShow: (id) sender;
+- (void) setUseIncompleteFolder: (id) sender;
+
+- (void) setRenamePartialFiles: (id) sender;
 
 - (void) applyRatioSetting: (id) sender;
-- (void) updateRatioStopField;
 - (void) setRatioStop: (id) sender;
+- (void) updateRatioStopField;
+
+- (void) applyIdleStopSetting: (id) sender;
+- (void) setIdleStop: (id) sender;
 
 - (void) applySpeedSettings: (id) sender;
+- (void) applyAltSpeedSettings;
 
 - (void) updateLimitFields;
 - (void) setGlobalLimit: (id) sender;
 
 - (void) setSpeedLimit: (id) sender;
 - (void) setAutoSpeedLimit: (id) sender;
+- (void) setAutoSpeedLimitTime: (id) sender;
+- (void) setAutoSpeedLimitDay: (id) sender;
++ (NSInteger) dateToTimeSum: (NSDate *) date;
++ (NSDate *) timeSumToDate: (NSInteger) sum;
 
 - (void) setAutoImport: (id) sender;
 - (void) importFolderSheetShow: (id) sender;
 
 - (void) setAutoSize: (id) sender;
 
-- (void) setProxyEnabled: (id) sender;
-- (void) setProxyAddress: (id) sender;
-- (void) setProxyPort: (id) sender;
-- (void) setProxyType: (id) sender;
-- (void) updateProxyType;
-- (void) setProxyAuthorize: (id) sender;
-- (void) setProxyUsername: (id) sender;
-- (void) setProxyPassword: (id) sender;
-- (void) updateProxyPassword;
-
 - (void) setRPCEnabled: (id) sender;
 - (void) linkWebUI: (id) sender;
 - (void) setRPCAuthorize: (id) sender;
@@ -133,6 +145,7 @@
 - (void) updateRPCPassword;
 - (void) setRPCPort: (id) sender;
 - (void) setRPCUseWhitelist: (id) sender;
+- (void) setRPCWebUIDiscovery: (id) sender;
 - (void) updateRPCWhitelist;
 - (void) addRemoveRPCIP: (id) sender;
 
