@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: PrefsController.m 10833 2010-06-24 01:19:29Z livings124 $
+ * $Id: PrefsController.m 11385 2010-11-05 22:45:38Z livings124 $
  *
  * Copyright (c) 2005-2010 Transmission authors and contributors
  *
@@ -39,10 +39,6 @@
 #define DOWNLOAD_FOLDER     0
 #define DOWNLOAD_TORRENT    2
 
-#define PROXY_HTTP      0
-#define PROXY_SOCKS4    1
-#define PROXY_SOCKS5    2
-
 #define RPC_IP_ADD_TAG      0
 #define RPC_IP_REMOVE_TAG   1
 
@@ -54,9 +50,6 @@
 #define TOOLBAR_NETWORK     @"TOOLBAR_NETWORK"
 #define TOOLBAR_REMOTE      @"TOOLBAR_REMOTE"
 
-#define PROXY_KEYCHAIN_SERVICE  "Transmission:Proxy"
-#define PROXY_KEYCHAIN_NAME     "Proxy"
-
 #define RPC_KEYCHAIN_SERVICE    "Transmission:Remote"
 #define RPC_KEYCHAIN_NAME       "Remote"
 
@@ -103,6 +96,21 @@ tr_session * fHandle;
             [fDefaults removeObjectForKey: @"DownloadChoice"];
         }
         
+        //check for old version blocklist (before 2.12)
+        NSDate * blocklistDate;
+        if ((blocklistDate = [fDefaults objectForKey: @"BlocklistLastUpdate"]))
+        {
+            [fDefaults setObject: blocklistDate forKey: @"BlocklistNewLastUpdateSuccess"];
+            [fDefaults setObject: blocklistDate forKey: @"BlocklistNewLastUpdate"];
+            [fDefaults removeObjectForKey: @"BlocklistLastUpdate"];
+            
+            NSString * blocklistDir = [NSHomeDirectory() stringByAppendingPathComponent:
+                                        @"/Library/Application Support/Transmission/blocklists/"];
+            [[NSFileManager defaultManager] moveItemAtPath: [blocklistDir stringByAppendingPathComponent: @"level1.bin"]
+                toPath: [blocklistDir stringByAppendingPathComponent: [NSString stringWithUTF8String: DEFAULT_BLOCKLIST_FILENAME]]
+                error: nil];
+        }
+        
         //save a new random port
         if ([fDefaults boolForKey: @"RandomPort"])
             [fDefaults setInteger: tr_sessionGetPeerPort(fHandle) forKey: @"BindPort"];
@@ -118,10 +126,6 @@ tr_session * fHandle;
         //set encryption
         [self setEncryptionMode: nil];
         
-        //set proxy type
-        [self updateProxyType];
-        [self updateProxyPassword];
-        
         //update rpc whitelist
         [self updateRPCPassword];
         
@@ -183,6 +187,9 @@ tr_session * fHandle;
     //set stop ratio
     [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
     
+    //set idle seeding minutes
+    [fIdleStopField setIntegerValue: [fDefaults integerForKey: @"IdleLimitMinutes"]];
+    
     //set limits
     [self updateLimitFields];
     
@@ -207,31 +214,6 @@ tr_session * fHandle;
     [fQueueSeedField setIntValue: [fDefaults integerForKey: @"QueueSeedNumber"]];
     [fStalledField setIntValue: [fDefaults integerForKey: @"StalledMinutes"]];
     
-    //set proxy type
-    [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
-    NSInteger proxyType;
-    switch (tr_sessionGetProxyType(fHandle))
-    {
-        case TR_PROXY_SOCKS4:
-            proxyType = PROXY_SOCKS4;
-            break;
-        case TR_PROXY_SOCKS5:
-            proxyType = PROXY_SOCKS5;
-            break;
-        case TR_PROXY_HTTP:
-            proxyType = PROXY_HTTP;
-            break;
-        default:
-            NSAssert(NO, @"Unknown proxy type received");
-    }
-    [fProxyTypePopUp selectItemAtIndex: proxyType];
-    
-    //set proxy password - does NOT need to be released
-    [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: tr_sessionGetProxyPassword(fHandle)]];
-    
-    //set proxy port
-    [fProxyPortField setIntValue: [fDefaults integerForKey: @"ProxyPort"]];
-    
     //set blocklist
     [self updateBlocklistFields];
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBlocklistFields)
@@ -495,13 +477,16 @@ tr_session * fHandle;
 
 - (void) setBlocklistEnabled: (id) sender
 {
-    const BOOL enable = [sender state] == NSOnState;
-    [fDefaults setBool: enable forKey: @"Blocklist"];
-    tr_blocklistSetEnabled(fHandle, enable);
+    tr_blocklistSetEnabled(fHandle, [fDefaults boolForKey: @"BlocklistNew"]);
     
     [[BlocklistScheduler scheduler] updateSchedule];
 }
 
+- (void) setBlocklistURL: (id) sender
+{
+    tr_blocklistSetURL(fHandle, [[fDefaults stringForKey: @"BlocklistURL"] UTF8String]);
+}
+
 - (void) updateBlocklist: (id) sender
 {
     [BlocklistDownloaderViewController downloadWithPrefsController: self];
@@ -531,13 +516,11 @@ tr_session * fHandle;
         [fBlocklistMessageField setStringValue: NSLocalizedString(@"A blocklist must first be downloaded",
             "Prefs -> blocklist -> message")];
     
-    [fBlocklistEnableCheck setEnabled: exists];
-    [fBlocklistEnableCheck setState: exists && [fDefaults boolForKey: @"Blocklist"]];
-    
     NSString * updatedDateString;
     if (exists)
     {
-        NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistLastUpdate"];
+        NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistNewLastUpdateSuccess"];
+        
         if (updatedDate)
         {
             if ([NSApp isOnSnowLeopardOrBetter])
@@ -571,29 +554,35 @@ tr_session * fHandle;
 - (void) applySpeedSettings: (id) sender
 {
     tr_sessionLimitSpeed(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
-    tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
+    tr_sessionSetSpeedLimit_KBps(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
     
     tr_sessionLimitSpeed(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
-    tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
+    tr_sessionSetSpeedLimit_KBps(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 - (void) applyAltSpeedSettings
 {
-    tr_sessionSetAltSpeed(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
-    tr_sessionSetAltSpeed(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
+    tr_sessionSetAltSpeed_KBps(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
+    tr_sessionSetAltSpeed_KBps(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
         
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 - (void) applyRatioSetting: (id) sender
 {
-    //[[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
     tr_sessionSetRatioLimited(fHandle, [fDefaults boolForKey: @"RatioCheck"]);
     tr_sessionSetRatioLimit(fHandle, [fDefaults floatForKey: @"RatioLimit"]);
 }
 
+- (void) setRatioStop: (id) sender
+{
+    [fDefaults setFloat: [sender floatValue] forKey: @"RatioLimit"];
+    
+    [self applyRatioSetting: nil];
+}
+
 - (void) updateRatioStopField
 {
     if (fHasLoaded)
@@ -602,11 +591,17 @@ tr_session * fHandle;
     [self applyRatioSetting: nil];
 }
 
-- (void) setRatioStop: (id) sender
+- (void) applyIdleStopSetting: (id) sender
 {
-    [fDefaults setFloat: [sender floatValue] forKey: @"RatioLimit"];
+    tr_sessionSetIdleLimited(fHandle, [fDefaults boolForKey: @"IdleLimitCheck"]);
+    tr_sessionSetIdleLimit(fHandle, [fDefaults integerForKey: @"IdleLimitMinutes"]);
+}
+
+- (void) setIdleStop: (id) sender
+{
+    [fDefaults setInteger: [sender integerValue] forKey: @"IdleLimitMinutes"];
     
-    [self applyRatioSetting: nil];
+    [self applyIdleStopSetting: nil];
 }
 
 - (void) updateLimitFields
@@ -814,107 +809,6 @@ tr_session * fHandle;
     [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoSizeSettingChange" object: self];
 }
 
-- (void) setProxyEnabled: (id) sender
-{
-    tr_sessionSetProxyEnabled(fHandle, [fDefaults boolForKey: @"Proxy"]);
-}
-
-- (void) setProxyAddress: (id) sender
-{
-    NSString * address = [sender stringValue];
-    tr_sessionSetProxy(fHandle, [address UTF8String]);
-    [fDefaults setObject: address forKey: @"ProxyAddress"];
-}
-
-- (void) setProxyPort: (id) sender
-{
-    int port = [sender intValue];
-    [fDefaults setInteger: port forKey: @"ProxyPort"];
-    tr_sessionSetProxyPort(fHandle, port);
-}
-
-- (void) setProxyType: (id) sender
-{
-    NSString * type;
-    switch ([sender indexOfSelectedItem])
-    {
-        case PROXY_HTTP:
-            type = @"HTTP";
-            break;
-        case PROXY_SOCKS4:
-            type = @"SOCKS4";
-            break;
-        case PROXY_SOCKS5:
-            type = @"SOCKS5";
-            break;
-        default:
-            NSAssert1(NO, @"Unknown index %d received for proxy type", [sender indexOfSelectedItem]);
-    }
-    
-    [fDefaults setObject: type forKey: @"ProxyType"];
-    [self updateProxyType];
-}
-
-- (void) updateProxyType
-{
-    NSString * typeString = [fDefaults stringForKey: @"ProxyType"];
-    tr_proxy_type type;
-    if ([typeString isEqualToString: @"SOCKS4"])
-        type = TR_PROXY_SOCKS4;
-    else if ([typeString isEqualToString: @"SOCKS5"])
-        type = TR_PROXY_SOCKS5;
-    else
-    {
-        //safety
-        if (![typeString isEqualToString: @"HTTP"])
-        {
-            typeString = @"HTTP";
-            [fDefaults setObject: typeString forKey: @"ProxyType"];
-        }
-        type = TR_PROXY_HTTP;
-    }
-    
-    tr_sessionSetProxyType(fHandle, type);
-}
-
-- (void) setProxyAuthorize: (id) sender
-{
-    BOOL enable = [fDefaults boolForKey: @"ProxyAuthorize"];
-    tr_sessionSetProxyAuthEnabled(fHandle, enable);
-}
-
-- (void) setProxyUsername: (id) sender
-{
-    tr_sessionSetProxyUsername(fHandle, [[fDefaults stringForKey: @"ProxyUsername"] UTF8String]);
-}
-
-- (void) setProxyPassword: (id) sender
-{
-    const char * password = [[sender stringValue] UTF8String];
-    [self setKeychainPassword: password forService: PROXY_KEYCHAIN_SERVICE username: PROXY_KEYCHAIN_NAME];
-    
-    tr_sessionSetProxyPassword(fHandle, password);
-}
-
-- (void) updateProxyPassword
-{
-    UInt32 passwordLength;
-    const char * password = nil;
-    SecKeychainFindGenericPassword(NULL, strlen(PROXY_KEYCHAIN_SERVICE), PROXY_KEYCHAIN_SERVICE,
-        strlen(PROXY_KEYCHAIN_NAME), PROXY_KEYCHAIN_NAME, &passwordLength, (void **)&password, NULL);
-    
-    if (password != NULL)
-    {
-        char fullPassword[passwordLength+1];
-        strncpy(fullPassword, password, passwordLength);
-        fullPassword[passwordLength] = '\0';
-        SecKeychainItemFreeContent(NULL, (void *)password);
-        
-        tr_sessionSetProxyPassword(fHandle, fullPassword);
-        [fProxyPasswordField setStringValue: [NSString stringWithUTF8String: fullPassword]];
-    }
-}
-
 - (void) setRPCEnabled: (id) sender
 {
     BOOL enable = [fDefaults boolForKey: @"RPC"];
@@ -1156,7 +1050,7 @@ tr_session * fHandle;
     
     //dht
     const BOOL lpd = tr_sessionIsLPDEnabled(fHandle);
-    [fDefaults setBool: lpd forKey: @"LocalPeerDiscovery"];
+    [fDefaults setBool: lpd forKey: @"LocalPeerDiscoveryGlobal"];
     
     //auto start
     const BOOL autoStart = !tr_sessionGetPaused(fHandle);
@@ -1180,14 +1074,14 @@ tr_session * fHandle;
     const BOOL downLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_DOWN);
     [fDefaults setBool: downLimitEnabled forKey: @"CheckDownload"];
     
-    const int downLimit = tr_sessionGetSpeedLimit(fHandle, TR_DOWN);
+    const int downLimit = tr_sessionGetSpeedLimit_KBps(fHandle, TR_DOWN);
     [fDefaults setInteger: downLimit forKey: @"DownloadLimit"];
     
     //speed limit - up
     const BOOL upLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_UP);
     [fDefaults setBool: upLimitEnabled forKey: @"CheckUpload"];
     
-    const int upLimit = tr_sessionGetSpeedLimit(fHandle, TR_UP);
+    const int upLimit = tr_sessionGetSpeedLimit_KBps(fHandle, TR_UP);
     [fDefaults setInteger: upLimit forKey: @"UploadLimit"];
     
     //alt speed limit enabled
@@ -1195,11 +1089,11 @@ tr_session * fHandle;
     [fDefaults setBool: useAltSpeed forKey: @"SpeedLimit"];
     
     //alt speed limit - down
-    const int downLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_DOWN);
+    const int downLimitAlt = tr_sessionGetAltSpeed_KBps(fHandle, TR_DOWN);
     [fDefaults setInteger: downLimitAlt forKey: @"SpeedLimitDownloadLimit"];
     
     //alt speed limit - up
-    const int upLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_UP);
+    const int upLimitAlt = tr_sessionGetAltSpeed_KBps(fHandle, TR_UP);
     [fDefaults setInteger: upLimitAlt forKey: @"SpeedLimitUploadLimit"];
     
     //alt speed limit schedule
@@ -1217,7 +1111,10 @@ tr_session * fHandle;
     
     //blocklist
     const BOOL blocklist = tr_blocklistIsEnabled(fHandle);
-    [fDefaults setBool: blocklist forKey: @"Blocklist"];
+    [fDefaults setBool: blocklist forKey: @"BlocklistNew"];
+    
+    NSString * blocklistURL = [NSString stringWithUTF8String: tr_blocklistGetURL(fHandle)];
+    [fDefaults setObject: blocklistURL forKey: @"BlocklistURL"];
     
     //seed ratio
     const BOOL ratioLimited = tr_sessionIsRatioLimited(fHandle);
@@ -1226,6 +1123,13 @@ tr_session * fHandle;
     const float ratioLimit = tr_sessionGetRatioLimit(fHandle);
     [fDefaults setFloat: ratioLimit forKey: @"RatioLimit"];
     
+    //Idle seed limit
+    const BOOL idleLimited = tr_sessionIsIdleLimited(fHandle);
+    [fDefaults setBool: idleLimited forKey: @"IdleLimitCheck"];
+    
+    const NSUInteger idleLimitMin = tr_sessionGetIdleLimit(fHandle);
+    [fDefaults setInteger: idleLimitMin forKey: @"IdleLimitMinutes"];
+    
     //update gui if loaded
     if (fHasLoaded)
     {
@@ -1264,6 +1168,9 @@ tr_session * fHandle;
         
         //ratio limit enabled handled by bindings
         [fRatioStopField setFloatValue: ratioLimit];
+        
+        //idle limit enabled handled by bindings
+        [fIdleStopField setIntegerValue: idleLimitMin];
     }
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
