@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.m 10878 2010-06-26 18:52:05Z livings124 $
+ * $Id: Torrent.m 11409 2010-11-14 15:16:39Z livings124 $
  *
  * Copyright (c) 2006-2010 Transmission authors and contributors
  *
@@ -31,22 +31,29 @@
 #import "transmission.h" // required by utils.h
 #import "utils.h" // tr_new()
 
+#define ETA_IDLE_DISPLAY_SEC (2*60)
+
 @interface Torrent (Private)
 
 - (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct
         magnetAddress: (NSString *) magnetAddress lib: (tr_session *) lib
         waitToStart: (NSNumber *) waitToStart
         groupValue: (NSNumber *) groupValue
-        downloadFolder: (NSString *) downloadFolder legacyIncompleteFolder: (NSString *) incompleteFolder;
+        timeMachineExcludeLocation: (NSString *) timeMachineExclude
+        downloadFolder: (NSString *) downloadFolder
+        legacyIncompleteFolder: (NSString *) incompleteFolder;
 
 - (void) createFileList;
 - (void) insertPath: (NSMutableArray *) components forParent: (FileListNode *) parent fileSize: (uint64_t) size
     index: (NSInteger) index flatList: (NSMutableArray *) flatFileList;
+- (void) sortFileList: (NSMutableArray *) fileNodes;
 
 - (void) completenessChange: (NSDictionary *) statusInfo;
 - (void) ratioLimitHit;
+- (void) idleLimitHit;
 - (void) metadataRetrieved;
 
+- (BOOL) shouldShowEta;
 - (NSString *) etaString;
 
 - (void) setTimeMachineExclude: (BOOL) exclude forPath: (NSString *) path;
@@ -65,6 +72,11 @@ void ratioLimitHitCallback(tr_torrent * torrent, void * torrentData)
     [(Torrent *)torrentData performSelectorOnMainThread: @selector(ratioLimitHit) withObject: nil waitUntilDone: NO];
 }
 
+void idleLimitHitCallback(tr_torrent * torrent, void * torrentData)
+{
+    [(Torrent *)torrentData performSelectorOnMainThread: @selector(idleLimitHit) withObject: nil waitUntilDone: NO];
+}
+
 void metadataCallback(tr_torrent * torrent, void * torrentData)
 {
     [(Torrent *)torrentData performSelectorOnMainThread: @selector(metadataRetrieved) withObject: nil waitUntilDone: NO];
@@ -84,7 +96,9 @@ int trashDataFile(const char * filename)
 {
     self = [self initWithPath: path hash: nil torrentStruct: NULL magnetAddress: nil lib: lib
             waitToStart: nil groupValue: nil
-            downloadFolder: location legacyIncompleteFolder: nil];
+            timeMachineExcludeLocation: nil
+            downloadFolder: location
+            legacyIncompleteFolder: nil];
     
     if (self)
     {
@@ -98,7 +112,9 @@ int trashDataFile(const char * filename)
 {
     self = [self initWithPath: nil hash: nil torrentStruct: torrentStruct magnetAddress: nil lib: lib
             waitToStart: nil groupValue: nil
-            downloadFolder: location legacyIncompleteFolder: nil];
+            timeMachineExcludeLocation: nil
+            downloadFolder: location
+            legacyIncompleteFolder: nil];
     
     return self;
 }
@@ -107,6 +123,7 @@ int trashDataFile(const char * filename)
 {
     self = [self initWithPath: nil hash: nil torrentStruct: nil magnetAddress: address
             lib: lib waitToStart: nil groupValue: nil
+            timeMachineExcludeLocation: nil
             downloadFolder: location legacyIncompleteFolder: nil];
     
     return self;
@@ -121,6 +138,7 @@ int trashDataFile(const char * filename)
                 lib: lib
                 waitToStart: [history objectForKey: @"WaitToStart"]
                 groupValue: [history objectForKey: @"GroupValue"]
+                timeMachineExcludeLocation: [history objectForKey: @"TimeMachineExcludeLocation"]
                 downloadFolder: [history objectForKey: @"DownloadFolder"] //upgrading from versions < 1.80
                 legacyIncompleteFolder: [[history objectForKey: @"UseIncompleteFolder"] boolValue] //upgrading from versions < 1.80
                                         ? [history objectForKey: @"IncompleteFolder"] : nil];
@@ -164,12 +182,17 @@ int trashDataFile(const char * filename)
 
 - (NSDictionary *) history
 {
-    return [NSDictionary dictionaryWithObjectsAndKeys:
-                [self torrentLocation], @"InternalTorrentPath",
-                [self hashString], @"TorrentHash",
-                [NSNumber numberWithBool: [self isActive]], @"Active",
-                [NSNumber numberWithBool: fWaitToStart], @"WaitToStart",
-                [NSNumber numberWithInt: fGroupValue], @"GroupValue", nil];
+    NSMutableDictionary * history = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                        [self torrentLocation], @"InternalTorrentPath",
+                                        [self hashString], @"TorrentHash",
+                                        [NSNumber numberWithBool: [self isActive]], @"Active",
+                                        [NSNumber numberWithBool: fWaitToStart], @"WaitToStart",
+                                        [NSNumber numberWithInt: fGroupValue], @"GroupValue", nil];
+    
+    if (fTimeMachineExclude)
+        [history setObject: fTimeMachineExclude forKey: @"TimeMachineExcludeLocation"];
+    
+    return history;
 }
 
 - (void) dealloc
@@ -206,7 +229,7 @@ int trashDataFile(const char * filename)
 
 - (void) closeRemoveTorrent
 {
-    //allow the file to be index by Time Machine
+    //allow the file to be indexed by Time Machine
     if (fTimeMachineExclude)
     {
         [self setTimeMachineExclude: NO forPath: fTimeMachineExclude];
@@ -292,7 +315,7 @@ int trashDataFile(const char * filename)
         tr_torrentStart(fHandle);
         [self update];
         
-        //capture, specifically, ratio setting changing to unlimited
+        //capture, specifically, stop-seeding settings changing to unlimited
         [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
     }
 }
@@ -377,16 +400,32 @@ int trashDataFile(const char * filename)
     tr_torrentSetRatioLimit(fHandle, limit);
 }
 
-- (BOOL) seedRatioSet
-{
-    return tr_torrentGetSeedRatio(fHandle, NULL);
-}
-
 - (CGFloat) progressStopRatio
 {
     return fStat->seedRatioPercentDone;
 }
 
+- (tr_idlelimit) idleSetting
+{
+    return tr_torrentGetIdleMode(fHandle);
+}
+
+- (void) setIdleSetting: (tr_idlelimit) setting
+{
+    tr_torrentSetIdleMode(fHandle, setting);
+}
+
+- (NSUInteger) idleLimitMinutes
+{
+    return tr_torrentGetIdleLimit(fHandle);
+}
+
+- (void) setIdleLimitMinutes: (NSUInteger) limit
+{
+    NSAssert(limit > 0, @"Idle limit must be greater than zero");
+    tr_torrentSetIdleLimit(fHandle, limit);
+}
+
 - (BOOL) usesSpeedLimit: (BOOL) upload
 {
     return tr_torrentUsesSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN);
@@ -399,12 +438,12 @@ int trashDataFile(const char * filename)
 
 - (NSInteger) speedLimit: (BOOL) upload
 {
-    return tr_torrentGetSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN);
+    return tr_torrentGetSpeedLimit_KBps(fHandle, upload ? TR_UP : TR_DOWN);
 }
 
 - (void) setSpeedLimit: (NSInteger) limit upload: (BOOL) upload
 {
-    tr_torrentSetSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN, limit);
+    tr_torrentSetSpeedLimit_KBps(fHandle, upload ? TR_UP : TR_DOWN, limit);
 }
 
 - (BOOL) usesGlobalSpeedLimit
@@ -789,11 +828,6 @@ int trashDataFile(const char * filename)
     return fStat->recheckProgress;
 }
 
-- (NSInteger) eta
-{
-    return fStat->eta;
-}
-
 - (CGFloat) availableDesired
 {
     return (CGFloat)fStat->desiredAvailable / [self sizeLeft];
@@ -883,9 +917,9 @@ int trashDataFile(const char * filename)
         [dict setObject: [NSString stringWithUTF8String: peer->flagStr] forKey: @"Flags"];
         
         if (peer->isUploadingTo)
-            [dict setObject: [NSNumber numberWithFloat: peer->rateToPeer] forKey: @"UL To Rate"];
+            [dict setObject: [NSNumber numberWithDouble: peer->rateToPeer_KBps] forKey: @"UL To Rate"];
         if (peer->isDownloadingFrom)
-            [dict setObject: [NSNumber numberWithFloat: peer->rateToClient] forKey: @"DL From Rate"];
+            [dict setObject: [NSNumber numberWithDouble: peer->rateToClient_KBps] forKey: @"DL From Rate"];
         
         [peerDicts addObject: dict];
     }
@@ -904,7 +938,7 @@ int trashDataFile(const char * filename)
 {
     NSMutableArray * webSeeds = [NSMutableArray arrayWithCapacity: fInfo->webseedCount];
     
-    float * dlSpeeds = tr_torrentWebSpeeds(fHandle);
+    double * dlSpeeds = tr_torrentWebSpeeds_KBps(fHandle);
     
     for (NSInteger i = 0; i < fInfo->webseedCount; i++)
     {
@@ -914,7 +948,7 @@ int trashDataFile(const char * filename)
         [dict setObject: [NSString stringWithUTF8String: fInfo->webseeds[i]] forKey: @"Address"];
         
         if (dlSpeeds[i] != -1.0)
-            [dict setObject: [NSNumber numberWithFloat: dlSpeeds[i]] forKey: @"DL From Rate"];
+            [dict setObject: [NSNumber numberWithDouble: dlSpeeds[i]] forKey: @"DL From Rate"];
         
         [webSeeds addObject: dict];
     }
@@ -929,8 +963,8 @@ int trashDataFile(const char * filename)
     if ([self isMagnet])
     {
         NSString * progressString = fStat->metadataPercentComplete > 0.0
-                    ? [NSString localizedStringWithFormat: NSLocalizedString(@"%.2f%% of torrent metadata retrieved",
-                        "Torrent -> progress string"), tr_truncd(100.0 * fStat->metadataPercentComplete, 2)]
+                    ? [NSString stringWithFormat: NSLocalizedString(@"%@ of torrent metadata retrieved",
+                        "Torrent -> progress string"), [NSString percentString: fStat->metadataPercentComplete longDecimals: YES]]
                     : NSLocalizedString(@"torrent metadata needed", "Torrent -> progress string");
         
         return [NSString stringWithFormat: @"%@ - %@", NSLocalizedString(@"Magnetized transfer", "Torrent -> progress string"),
@@ -946,16 +980,16 @@ int trashDataFile(const char * filename)
         {
             string = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@ selected", "Torrent -> progress string"),
                         [NSString stringForFileSize: [self haveTotal]], [NSString stringForFileSize: [self totalSizeSelected]]];
-            progress = 100.0 * [self progressDone];
+            progress = [self progressDone];
         }
         else
         {
             string = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@", "Torrent -> progress string"),
                         [NSString stringForFileSize: [self haveTotal]], [NSString stringForFileSize: [self size]]];
-            progress = 100.0 * [self progress];
+            progress = [self progress];
         }
         
-        string = [NSString localizedStringWithFormat: @"%@ (%.2f%%)", string, tr_truncd(progress, 2)];
+        string = [string stringByAppendingFormat: @" (%@)", [NSString percentString: progress longDecimals: YES]];
     }
     else
     {
@@ -969,9 +1003,8 @@ int trashDataFile(const char * filename)
             {
                 downloadString = [NSString stringWithFormat: NSLocalizedString(@"%@ of %@", "Torrent -> progress string"),
                                     [NSString stringForFileSize: [self haveTotal]], [NSString stringForFileSize: [self size]]];
-                
-                downloadString = [NSString localizedStringWithFormat: @"%@ (%.2f%%)",
-                                    downloadString, tr_truncd(100.0 * [self progress], 2)];
+                downloadString = [downloadString stringByAppendingFormat: @" (%@)",
+                                    [NSString percentString: [self progress] longDecimals: YES]];
             }
         }
         else
@@ -984,8 +1017,8 @@ int trashDataFile(const char * filename)
         string = [downloadString stringByAppendingFormat: @", %@", uploadString];
     }
     
-    //add time when downloading
-    if (fStat->activity == TR_STATUS_DOWNLOAD || ([self isSeeding] && [self seedRatioSet]))
+    //add time when downloading or seed limit set
+    if ([self shouldShowEta])
         string = [string stringByAppendingFormat: @" - %@", [self etaString]];
     
     return string;
@@ -1031,9 +1064,9 @@ int trashDataFile(const char * filename)
                 break;
 
             case TR_STATUS_CHECK:
-                string = [NSString localizedStringWithFormat: @"%@ (%.2f%%)",
+                string = [NSString stringWithFormat: @"%@ (%@)",
                             NSLocalizedString(@"Checking existing data", "Torrent -> status string"),
-                                tr_truncd(100.0 * [self checkingProgress], 2)];
+                            [NSString percentString: [self checkingProgress] longDecimals: YES]];
                 break;
 
             case TR_STATUS_DOWNLOAD:
@@ -1111,9 +1144,9 @@ int trashDataFile(const char * filename)
             break;
 
         case TR_STATUS_CHECK:
-            string = [NSString localizedStringWithFormat: @"%@ (%.2f%%)",
+            string = [NSString stringWithFormat: @"%@ (%@)",
                         NSLocalizedString(@"Checking existing data", "Torrent -> status string"),
-                            tr_truncd(100.0 * [self checkingProgress], 2)];
+                        [NSString percentString: [self checkingProgress] longDecimals: YES]];
             break;
         
         case TR_STATUS_DOWNLOAD:
@@ -1133,7 +1166,7 @@ int trashDataFile(const char * filename)
 
 - (NSString *) remainingTimeString
 {
-    if (fStat->activity == TR_STATUS_DOWNLOAD || ([self isSeeding] && [self seedRatioSet]))
+    if ([self shouldShowEta])
         return [self etaString];
     else
         return [self shortStatusString];
@@ -1165,9 +1198,9 @@ int trashDataFile(const char * filename)
             return [NSLocalizedString(@"Waiting to check existing data", "Torrent -> status string") stringByAppendingEllipsis];
 
         case TR_STATUS_CHECK:
-            return [NSString localizedStringWithFormat: @"%@ (%.2f%%)",
+            return [NSString stringWithFormat: @"%@ (%@)",
                     NSLocalizedString(@"Checking existing data", "Torrent -> status string"),
-                        tr_truncd(100.0 * [self checkingProgress], 2)];
+                    [NSString percentString: [self checkingProgress] longDecimals: YES]];
 
         case TR_STATUS_DOWNLOAD:
             return NSLocalizedString(@"Downloading", "Torrent -> status string");
@@ -1234,12 +1267,12 @@ int trashDataFile(const char * filename)
 
 - (CGFloat) downloadRate
 {
-    return fStat->pieceDownloadSpeed;
+    return fStat->pieceDownloadSpeed_KBps;
 }
 
 - (CGFloat) uploadRate
 {
-    return fStat->pieceUploadSpeed;
+    return fStat->pieceUploadSpeed_KBps;
 }
 
 - (CGFloat) totalRate
@@ -1483,16 +1516,10 @@ int trashDataFile(const char * filename)
 
 - (NSInteger) stalledMinutes
 {
-    const time_t start = fStat->startDate;
-    if (start == 0)
+    if (fStat->idleSecs == -1)
         return -1;
     
-    NSDate * started = [NSDate dateWithTimeIntervalSince1970: start],
-            * activity = [self dateActivity];
-    
-    NSDate * laterDate = activity ? [started laterDate: activity] : started;
-    
-    return ABS([laterDate timeIntervalSinceNow]) / 60;
+    return fStat->idleSecs / 60;
 }
 
 - (BOOL) isStalled
@@ -1576,7 +1603,9 @@ int trashDataFile(const char * filename)
         magnetAddress: (NSString *) magnetAddress lib: (tr_session *) lib
         waitToStart: (NSNumber *) waitToStart
         groupValue: (NSNumber *) groupValue
-        downloadFolder: (NSString *) downloadFolder legacyIncompleteFolder: (NSString *) incompleteFolder
+        timeMachineExcludeLocation: (NSString *) timeMachineExclude
+        downloadFolder: (NSString *) downloadFolder
+        legacyIncompleteFolder: (NSString *) incompleteFolder
 {
     if (!(self = [super init]))
         return nil;
@@ -1623,6 +1652,7 @@ int trashDataFile(const char * filename)
     
     tr_torrentSetCompletenessCallback(fHandle, completenessChangeCallback, self);
     tr_torrentSetRatioLimitHitCallback(fHandle, ratioLimitHitCallback, self);
+    tr_torrentSetIdleLimitHitCallback(fHandle, idleLimitHitCallback, self);
     tr_torrentSetMetadataCallback(fHandle, metadataCallback, self);
     
     fHashString = [[NSString alloc] initWithUTF8String: fInfo->hashString];
@@ -1639,7 +1669,7 @@ int trashDataFile(const char * filename)
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(checkGroupValueForRemoval:)
         name: @"GroupValueRemoved" object: nil];
     
-    fTimeMachineExclude = nil;
+    fTimeMachineExclude = [timeMachineExclude retain];
     [self update];
     
     return self;
@@ -1696,6 +1726,9 @@ int trashDataFile(const char * filename)
             }
         }
         
+        [self sortFileList: fileList];
+        [self sortFileList: flatFileList];
+        
         fFileList = [[NSArray alloc] initWithArray: fileList];
         fFlatFileList = [[NSArray alloc] initWithArray: flatFileList];
     }
@@ -1747,6 +1780,17 @@ int trashDataFile(const char * filename)
     }
 }
 
+- (void) sortFileList: (NSMutableArray *) fileNodes
+{
+    NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"name" ascending: YES
+                                            selector: @selector(compareFinder:)] autorelease];
+    [fileNodes sortUsingDescriptors: [NSArray arrayWithObject: descriptor]];
+    
+    for (FileListNode * node in fileNodes)
+        if ([node isFolder])
+            [self sortFileList: [node children]];
+}
+
 //status has been retained
 - (void) completenessChange: (NSDictionary *) statusInfo
 {
@@ -1756,8 +1800,8 @@ int trashDataFile(const char * filename)
     {
         case TR_SEED:
         case TR_PARTIAL_SEED:
-            //simpler to create a new dictionary than to use statusInfo - avoids retention chicanery 
-            [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentFinishedDownloading" object: self 
+            //simpler to create a new dictionary than to use statusInfo - avoids retention chicanery
+            [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentFinishedDownloading" object: self
                 userInfo: [NSDictionary dictionaryWithObject: [statusInfo objectForKey: @"WasRunning"] forKey: @"WasRunning"]];
             break;
         
@@ -1775,7 +1819,14 @@ int trashDataFile(const char * filename)
 {
     fStat = tr_torrentStat(fHandle);
     
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentStoppedForRatio" object: self];
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentFinishedSeeding" object: self];
+}
+
+- (void) idleLimitHit
+{
+    fStat = tr_torrentStat(fHandle);
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentFinishedSeeding" object: self];
 }
 
 - (void) metadataRetrieved
@@ -1787,18 +1838,48 @@ int trashDataFile(const char * filename)
     [[NSNotificationCenter defaultCenter] postNotificationName: @"ResetInspector" object: self];
 }
 
+- (BOOL) shouldShowEta
+{
+    if (fStat->activity == TR_STATUS_DOWNLOAD)
+        return YES;
+    else if ([self isSeeding])
+    {
+        //ratio: show if it's set at all
+        if (tr_torrentGetSeedRatio(fHandle, NULL))
+            return YES;
+        
+        //idle: show only if remaining time is less than cap
+        if (fStat->etaIdle != TR_ETA_NOT_AVAIL && fStat->etaIdle < ETA_IDLE_DISPLAY_SEC)
+            return YES;
+    }
+    
+    return NO;
+}
+
 - (NSString *) etaString
 {
-    const NSInteger eta = [self eta];
-    switch (eta)
+    NSInteger eta;
+    BOOL fromIdle;
+    //don't check for both, since if there's a regular ETA, the torrent isn't idle so it's meaningless
+    if (fStat->eta != TR_ETA_NOT_AVAIL && fStat->eta != TR_ETA_UNKNOWN)
     {
-        case TR_ETA_NOT_AVAIL:
-        case TR_ETA_UNKNOWN:
-            return NSLocalizedString(@"remaining time unknown", "Torrent -> eta string");
-        default:
-            return [NSString stringWithFormat: NSLocalizedString(@"%@ remaining", "Torrent -> eta string"),
-                        [NSString timeString: eta showSeconds: YES maxFields: 2]];
+        eta = fStat->eta;
+        fromIdle = NO;
     }
+    else if (fStat->etaIdle != TR_ETA_NOT_AVAIL && fStat->etaIdle < ETA_IDLE_DISPLAY_SEC)
+    {
+        eta = fStat->etaIdle;
+        fromIdle = YES;
+    }
+    else
+        return NSLocalizedString(@"remaining time unknown", "Torrent -> eta string");
+    
+    NSString * idleString = [NSString stringWithFormat: NSLocalizedString(@"%@ remaining", "Torrent -> eta string"),
+                                [NSString timeString: eta showSeconds: YES maxFields: 2]];
+    if (fromIdle)
+        idleString = [idleString stringByAppendingFormat: @" (%@)", NSLocalizedString(@"inactive", "Torrent -> eta string")];
+    
+    return idleString;
 }
 
 - (void) setTimeMachineExclude: (BOOL) exclude forPath: (NSString *) path
