@@ -1,21 +1,25 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: app.h 10775 2010-06-16 03:11:10Z Longinus00 $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: app.h 11094 2010-08-02 03:07:42Z charles $
  */
 
 #ifndef QTR_APP_H
 #define QTR_APP_H
 
 #include <QApplication>
+#include <QSet>
 #include <QTimer>
 
+#include "favicon.h"
+
+class AddData;
 class Prefs;
 class Session;
 class TorrentModel;
@@ -32,6 +36,10 @@ class MyApp: public QApplication
 
     public:
         void raise( );
+        bool notify( const QString& title, const QString& body ) const;
+
+    public:
+        Favicons favicons;
 
     private:
         Prefs * myPrefs;
@@ -49,9 +57,12 @@ class MyApp: public QApplication
         void onSessionSourceChanged( );
         void refreshPref( int key );
         void refreshTorrents( );
+        void onTorrentsAdded( QSet<int> );
+        void onNewTorrentChanged( int );
 
     public slots:
         void addTorrent( const QString& );
+        void addTorrent( const AddData& );
 
     private:
         void maybeUpdateBlocklist( );
