@@ -1,18 +1,17 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: details.cc 10952 2010-07-06 15:54:40Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: details.cc 11396 2010-11-08 19:16:03Z charles $
  */
 
 #include <cassert>
 #include <ctime>
-#include <iostream>
 
 #include <QCheckBox>
 #include <QComboBox>
@@ -25,12 +24,17 @@
 #include <QHBoxLayout>
 #include <QHBoxLayout>
 #include <QHeaderView>
+#include <QInputDialog>
+#include <QItemSelectionModel>
 #include <QLabel>
-#include <QLocale>
+#include <QList>
+#include <QMap>
+#include <QMessageBox>
 #include <QPushButton>
 #include <QRadioButton>
 #include <QResizeEvent>
 #include <QSpinBox>
+#include <QStringList>
 #include <QStyle>
 #include <QTabWidget>
 #include <QTextBrowser>
@@ -40,16 +44,21 @@
 #include <QVBoxLayout>
 
 #include <libtransmission/transmission.h>
+#include <libtransmission/bencode.h>
+#include <libtransmission/utils.h> // tr_getRatio()
 
 #include "details.h"
 #include "file-tree.h"
+#include "formatter.h"
 #include "hig.h"
 #include "prefs.h"
 #include "session.h"
 #include "squeezelabel.h"
 #include "torrent.h"
 #include "torrent-model.h"
-#include "utils.h"
+#include "tracker-delegate.h"
+#include "tracker-model.h"
+#include "tracker-model-filter.h"
 
 class Prefs;
 class Session;
@@ -118,6 +127,17 @@ class PeerItem: public QTreeWidgetItem
 ****
 ***/
 
+QIcon
+Details :: getStockIcon( const QString& freedesktop_name, int fallback )
+{
+    QIcon icon = QIcon::fromTheme( freedesktop_name );
+
+    if( icon.isNull( ) )
+        icon = style()->standardIcon( QStyle::StandardPixmap( fallback ), 0, this );
+
+    return icon;
+}
+
 Details :: Details( Session& session, Prefs& prefs, TorrentModel& model, QWidget * parent ):
     QDialog( parent, Qt::Dialog ),
     mySession( session ),
@@ -149,7 +169,14 @@ Details :: Details( Session& session, Prefs& prefs, TorrentModel& model, QWidget
     layout->addWidget( buttons );
     QWidget::setAttribute( Qt::WA_DeleteOnClose, true );
 
+    QList<int> initKeys;
+    initKeys << Prefs :: SHOW_TRACKER_SCRAPES
+             << Prefs :: SHOW_BACKUP_TRACKERS;
+    foreach( int key, initKeys )
+        refreshPref( key );
+
     connect( &myTimer, SIGNAL(timeout()), this, SLOT(onTimer()));
+    connect( &myPrefs, SIGNAL(changed(int)), this, SLOT(refreshPref(int)) );
 
     onTimer( );
     myTimer.setSingleShot( false );
@@ -158,6 +185,9 @@ Details :: Details( Session& session, Prefs& prefs, TorrentModel& model, QWidget
 
 Details :: ~Details( )
 {
+    myTrackerDelegate->deleteLater();
+    myTrackerFilter->deleteLater();
+    myTrackerModel->deleteLater();
 }
 
 void
@@ -176,8 +206,8 @@ Details :: setIds( const QSet<int>& ids )
     }
 
     myFileTreeView->clear( );
-
     myIds = ids;
+    myTrackerModel->refresh( myModel, myIds );
 
     // listen to the new torrents
     foreach( int id, myIds ) {
@@ -192,6 +222,35 @@ Details :: setIds( const QSet<int>& ids )
     onTimer( );
 }
 
+void
+Details :: refreshPref( int key )
+{
+    QString str;
+
+    switch( key )
+    {
+        case Prefs :: SHOW_TRACKER_SCRAPES: {
+            QItemSelectionModel * selectionModel( myTrackerView->selectionModel( ) );
+            const QItemSelection selection( selectionModel->selection( ) );
+            const QModelIndex currentIndex( selectionModel->currentIndex( ) );
+            myTrackerDelegate->setShowMore( myPrefs.getBool( key ) );
+            selectionModel->clear( );
+            myTrackerView->reset( );
+            selectionModel->select( selection, QItemSelectionModel::Select );
+            selectionModel->setCurrentIndex( currentIndex, QItemSelectionModel::NoUpdate );
+            break;
+        }
+
+        case Prefs :: SHOW_BACKUP_TRACKERS:
+            myTrackerFilter->setShowBackupTrackers( myPrefs.getBool( key ) );
+            break;
+
+        default:
+            break;
+    }
+}
+
+
 /***
 ****
 ***/
@@ -200,11 +259,17 @@ QString
 Details :: timeToStringRounded( int seconds )
 {
     if( seconds > 60 ) seconds -= ( seconds % 60 );
-    return Utils::timeToString ( seconds );
+    return Formatter::timeToString ( seconds );
 }
 
 void
 Details :: onTimer( )
+{
+    getNewData( );
+}
+
+void
+Details :: getNewData( )
 {
     if( !myIds.empty( ) )
     {
@@ -233,7 +298,6 @@ Details :: onTorrentChanged( )
 void
 Details :: refresh( )
 {
-    QLocale locale;
     const int n = myIds.size( );
     const bool single = n == 1;
     const QString blank;
@@ -306,29 +370,50 @@ Details :: refresh( )
                 available += t->sizeWhenDone() - t->leftUntilDone() + t->desiredAvailable();
             }
         }
-        if( !haveVerified && !haveUnverified )
-            string = none;
-        else {
+        {
             const double d = 100.0 * ( sizeWhenDone ? ( sizeWhenDone - leftUntilDone ) / sizeWhenDone : 1 );
-            QString pct = locale.toString( d, 'f', 2 );
-            if( !haveUnverified )
-                string = tr( "%1 (%2%)" )
-                             .arg( Utils :: sizeToString( haveVerified + haveUnverified ) )
-                             .arg( pct );
+            QString pct = Formatter::percentToString( d );
+            QString astr;
+
+            if( sizeWhenDone )
+                astr = Formatter::percentToString( ( 100.0 * available ) / sizeWhenDone );
             else
-                string = tr( "%1 (%2%); %3 Unverified" )
-                             .arg( Utils :: sizeToString( haveVerified + haveUnverified ) )
+                astr = "100";
+
+            if( !haveUnverified && !leftUntilDone )
+            {
+                string = tr( "%1 (%2%)" )
+                             .arg( Formatter::sizeToString( haveVerified + haveUnverified ) )
+                             .arg( pct );
+            }
+            else if( !haveUnverified )
+            {
+                string = tr( "%1 (%2% of %3% Available)" )
+                             .arg( Formatter::sizeToString( haveVerified + haveUnverified ) )
                              .arg( pct )
-                             .arg( Utils :: sizeToString( haveUnverified ) );
+                             .arg( astr );
+            }
+            else
+            {
+                string = tr( "%1 (%2% of %3% Available) + %4 Unverified" )
+                             .arg( Formatter::sizeToString( haveVerified + haveUnverified ) )
+                             .arg( pct )
+                             .arg( astr )
+                             .arg( Formatter::sizeToString( haveUnverified ) );
+            }
         }
     }
     myHaveLabel->setText( string );
 
     // myAvailabilityLabel
-    if( sizeWhenDone < 1 )
+    if( torrents.empty( ) )
         string = none;
-    else
-        string.sprintf( "%'.1f%%", ( 100.0 * available ) / sizeWhenDone );
+    else {
+        if( sizeWhenDone == 0 )
+            string = none;
+        else
+            string = QString( "%1%" ).arg( Formatter::percentToString( ( 100.0 * available ) / sizeWhenDone ) );
+    }
     myAvailabilityLabel->setText( string );
 
     // myDownloadedLabel
@@ -340,8 +425,8 @@ Details :: refresh( )
             d += t->downloadedEver( );
             f += t->failedEver( );
         }
-        const QString dstr = Utils::sizeToString( d );
-        const QString fstr = Utils::sizeToString( f );
+        const QString dstr = Formatter::sizeToString( d );
+        const QString fstr = Formatter::sizeToString( f );
         if( f )
             string = tr( "%1 (+%2 corrupt)" ).arg( dstr ).arg( fstr );
         else
@@ -349,40 +434,21 @@ Details :: refresh( )
     }
     myDownloadedLabel->setText( string );
 
-    uint64_t u = 0;
     if( torrents.empty( ) )
         string = none;
     else {
-        foreach( const Torrent * t, torrents ) u += t->uploadedEver( );
-        string = QString( Utils::sizeToString( u ) );
+        uint64_t u = 0;
+        uint64_t d = 0;
+        foreach( const Torrent * t, torrents ) {
+            u += t->uploadedEver( );
+            d += t->downloadedEver( );
+        }
+        string = tr( "%1 (Ratio: %2)" )
+                   .arg( Formatter::sizeToString( u ) )
+                   .arg( Formatter::ratioToString( tr_getRatio( u, d ) ) );
     }
     myUploadedLabel->setText( string );
 
-    if( torrents.empty( ) )
-        string = none;
-    else if( torrents.count() == 1 )
-        string = QString( Utils :: ratioToString( torrents.first()->ratio() ) );
-    else {
-        bool isMixed = false;
-        int ratioType = torrents.first()->ratio();
-        if( ratioType > 0 ) ratioType = 0;
-        foreach( const Torrent *t, torrents )
-        {
-            if( ratioType != ( t->ratio() >= 0 ? 0 : t->ratio() ) )
-            {
-                isMixed = true;
-                break;
-            }
-        }
-        if( isMixed )
-            string = mixed;
-        else if( ratioType < 0 )
-            string = QString( Utils :: ratioToString( ratioType ) );
-        else
-            string = QString( Utils :: ratioToString( (double)u / d ) );
-    }
-    myRatioLabel->setText( string );
-
     const QDateTime qdt_now = QDateTime::currentDateTime( );
 
     // myRunTimeLabel
@@ -402,7 +468,7 @@ Details :: refresh( )
         else if( baseline.isNull( ) )
             string = mixed;
         else
-            string = Utils::timeToString( baseline.secsTo( qdt_now ) );
+            string = Formatter::timeToString( baseline.secsTo( qdt_now ) );
     }
     myRunTimeLabel->setText( string );
 
@@ -423,7 +489,7 @@ Details :: refresh( )
             if( baseline < 0 )
                 string = tr( "Unknown" );
             else
-                string = Utils::timeToString( baseline );
+                string = Formatter::timeToString( baseline );
        }
     }
     myETALabel->setText( string );
@@ -439,11 +505,13 @@ Details :: refresh( )
             if( latest < dt )
                 latest = dt;
         }
-        const int seconds = latest.secsTo( qdt_now );
-        if( seconds < 5 )
+        const int seconds = latest.isValid() ? latest.secsTo( qdt_now ) : -1;
+        if( seconds < 0 )
+            string = none;
+        else if( seconds < 5 )
             string = tr( "Active now" );
         else
-            string = tr( "%1 ago" ).arg( Utils::timeToString( seconds ) );
+            string = tr( "%1 ago" ).arg( Formatter::timeToString( seconds ) );
     }
     myLastActivityLabel->setText( string );
 
@@ -485,11 +553,11 @@ Details :: refresh( )
             string = none;
         else if( pieceSize > 0 )
             string = tr( "%1 (%Ln pieces @ %2)", "", pieces )
-                     .arg( Utils::sizeToString( size ) )
-                     .arg( Utils::sizeToString( pieceSize ) );
+                     .arg( Formatter::sizeToString( size ) )
+                     .arg( Formatter::memToString( pieceSize ) );
         else
             string = tr( "%1 (%Ln pieces)", "", pieces )
-                     .arg( Utils::sizeToString( size ) );
+                     .arg( Formatter::sizeToString( size ) );
     }
     mySizeLabel->setText( string );
 
@@ -551,10 +619,12 @@ Details :: refresh( )
         }
         if( mixed_creator && mixed_date )
             string = mixed;
-        else if( mixed_date )
+        else if( mixed_date && !creator.isEmpty())
             string = tr( "Created by %1" ).arg( creator );
-        else if( mixed_creator || creator.isEmpty( ) )
+        else if( mixed_creator && !date.isEmpty())
             string = tr( "Created on %1" ).arg( date );
+        else if( creator.isEmpty() && date.isEmpty())
+            string = tr( "N/A" );
         else
             string = tr( "Created by %1 on %2" ).arg( creator ).arg( date );
     }
@@ -619,178 +689,56 @@ Details :: refresh( )
         myBandwidthPriorityCombo->blockSignals( false );
 
         mySingleDownSpin->blockSignals( true );
-        mySingleDownSpin->setValue( int(tor->downloadLimit().kbps()) );
+        mySingleDownSpin->setValue( (int)tor->downloadLimit().KBps() );
         mySingleDownSpin->blockSignals( false );
 
         mySingleUpSpin->blockSignals( true );
-        mySingleUpSpin->setValue( int(tor->uploadLimit().kbps()) );
+        mySingleUpSpin->setValue( (int)tor->uploadLimit().KBps() );
         mySingleUpSpin->blockSignals( false );
 
         myPeerLimitSpin->blockSignals( true );
         myPeerLimitSpin->setValue( tor->peerLimit() );
         myPeerLimitSpin->blockSignals( false );
-
-        // ratio radios
-        uniform = true;
-        baselineInt = tor->seedRatioMode( );
-        foreach( tor, torrents ) if( baselineInt != tor->seedRatioMode( ) ) { uniform = false; break; }
-        if( !uniform ) {
-            mySeedGlobalRadio->setChecked( false );
-            mySeedCustomRadio->setChecked( false );
-            mySeedForeverRadio->setChecked( false );
-        } else {
-            QRadioButton * rb;
-            switch( baselineInt ) {
-                case TR_RATIOLIMIT_GLOBAL:    rb = mySeedGlobalRadio; break;
-                case TR_RATIOLIMIT_SINGLE:    rb = mySeedCustomRadio; break;
-                case TR_RATIOLIMIT_UNLIMITED: rb = mySeedForeverRadio; break;
-            }
-            rb->setChecked( true );
-        }
-
-        mySeedCustomSpin->blockSignals( true );
-        mySeedCustomSpin->setValue( tor->seedRatioLimit( ) );
-        mySeedCustomSpin->blockSignals( false );
     }
 
-    // tracker tab
-    //
-    QMap<QString,QTreeWidgetItem*> trackers2;
-    QList<QTreeWidgetItem*> newItems2;
-    const time_t now( time( 0 ) );
-    const bool showBackup = myPrefs.getBool( Prefs::SHOW_BACKUP_TRACKERS );
-    const bool showScrape = myPrefs.getBool( Prefs::SHOW_TRACKER_SCRAPES );
-    foreach( const Torrent * t, torrents )
+    if( !torrents.empty( ) )
     {
-        const QString idStr( QString::number( t->id( ) ) );
-        TrackerStatsList trackerStats = t->trackerStats( );
+        const Torrent * tor;
 
-        foreach( const TrackerStat& trackerStat, trackerStats )
-        {
-            const QString key( idStr + ":" + QString::number(trackerStat.id) );
-            QTreeWidgetItem * item = (QTreeWidgetItem*) myTrackerStats.value( key, 0 );
-            QString str;
+        // ratio
+        bool uniform = true;
+        int baselineInt = torrents[0]->seedRatioMode( );
+        foreach( tor, torrents ) if( baselineInt != tor->seedRatioMode( ) ) { uniform = false; break; }
 
-            if( item == 0 ) // new tracker
-            {
-                item = new QTreeWidgetItem( myTrackerTree );
-                newItems2 << item;
-            }
-            str = trackerStat.host;
-            if( showBackup || !trackerStat.isBackup)
-            {
-                if( trackerStat.hasAnnounced )
-                {
-                    const QString tstr( timeToStringRounded( now - trackerStat.lastAnnounceTime ) );
-                    str += "\n";
-                    if( trackerStat.lastAnnounceSucceeded )
-                    {
-                        str += tr( "Got a list of %1 peers %2 ago" )
-                            .arg( trackerStat.lastAnnouncePeerCount )
-                            .arg( tstr );
-                    }
-                    else if( trackerStat.lastAnnounceTimedOut )
-                    {
-                        str += tr( "Peer list request timed out %1 ago; will retry" )
-                            .arg( tstr );
-                    }
-                    else
-                    {
-                        str += tr( "Got an error %1 ago" )
-                            .arg( tstr );
-                    }
-                }
-                switch( trackerStat.announceState )
-                {
-                    case TR_TRACKER_INACTIVE:
-                        if( trackerStat.hasAnnounced )
-                        {
-                            str += "\n";
-                            str += tr( "No updates scheduled" );
-                        }
-                        break;
-                    case TR_TRACKER_WAITING:
-                        {
-                            const QString tstr( timeToStringRounded( trackerStat.nextAnnounceTime - now ) );
-                            str += "\n";
-                            str += tr( "Asking for more peers in %1" )
-                                .arg( tstr );
-                        }
-                        break;
-                    case TR_TRACKER_QUEUED:
-                        str += "\n";
-                        str += tr( "Queued to ask for more peers" );
-                        break;
-                    case TR_TRACKER_ACTIVE:
-                        {
-                            const QString tstr( timeToStringRounded( now - trackerStat.lastAnnounceStartTime ) );
-                            str += "\n";
-                            str += tr( "Asking for more peers now... %1" )
-                                .arg( tstr );
-                        }
-                        break;
-                }
-                if( showScrape )
-                {
-                    if( trackerStat.hasScraped )
-                    {
-                        const QString tstr( timeToStringRounded( now - trackerStat.lastScrapeTime ) );
-                        str += "\n";
-                        if( trackerStat.lastScrapeSucceeded )
-                        {
-                            str += tr( "Tracker had %1 seeders and %2 leechers %3 ago" )
-                                .arg( trackerStat.seederCount )
-                                .arg( trackerStat.leecherCount )
-                                .arg( tstr );
-                        }
-                        else
-                        {
-                            str += tr( "Got a scrape error %1 ago" )
-                                .arg( tstr );
-                        }
-                    }
-                    switch( trackerStat.scrapeState )
-                    {
-                        case TR_TRACKER_INACTIVE:
-                            break;
-                        case TR_TRACKER_WAITING:
-                            {
-                                const QString tstr( timeToStringRounded( trackerStat.nextScrapeTime - now ) );
-                                str += "\n";
-                                str += tr( "Asking for peer counts in %1" )
-                                    .arg( tstr );
-                            }
-                            break;
-                        case TR_TRACKER_QUEUED:
-                            str += "\n";
-                            str += tr( "Queued to ask for peer counts" );
-                            break;
-                        case TR_TRACKER_ACTIVE:
-                            {
-                                const QString tstr( timeToStringRounded( now - trackerStat.lastScrapeStartTime ) );
-                                str += "\n";
-                                str += tr( "Asking for peer counts now... %1" )
-                                    .arg( tstr );
-                            }
-                            break;
-                    }
-                }
-            }
+        myRatioCombo->blockSignals( true );
+        myRatioCombo->setCurrentIndex( uniform ? myRatioCombo->findData( baselineInt ) : -1 );
+        myRatioSpin->setVisible( uniform && ( baselineInt == TR_RATIOLIMIT_SINGLE ) );
+        myRatioCombo->blockSignals( false );
 
-            item->setText( 0, str );
+        myRatioSpin->blockSignals( true );
+        myRatioSpin->setValue( tor->seedRatioLimit( ) );
+        myRatioSpin->blockSignals( false );
 
-            trackers2.insert( key, item );
-        }
+        // idle
+        uniform = true;
+        baselineInt = torrents[0]->seedIdleMode( );
+        foreach( tor, torrents ) if( baselineInt != tor->seedIdleMode( ) ) { uniform = false; break; }
+
+        myIdleCombo->blockSignals( true );
+        myIdleCombo->setCurrentIndex( uniform ? myIdleCombo->findData( baselineInt ) : -1 );
+        myIdleSpin->setVisible( uniform && ( baselineInt == TR_RATIOLIMIT_SINGLE ) );
+        myIdleCombo->blockSignals( false );
+
+        myIdleSpin->blockSignals( true );
+        myIdleSpin->setValue( tor->seedIdleLimit( ) );
+        myIdleSpin->blockSignals( false );
     }
-    myTrackerTree->addTopLevelItems( newItems2 );
-    foreach( QString key, myTrackerStats.keys() ) {
-        if( !trackers2.contains( key ) ) { // tracker has disappeared
-            QTreeWidgetItem * item = myTrackerStats.value( key, 0 );
-            myTrackerTree->takeTopLevelItem( myTrackerTree->indexOfTopLevelItem( item ) );
-            delete item;
-        }
-    }
-    myTrackerStats = trackers2;
+
+    ///
+    ///  Tracker tab
+    ///
+
+    myTrackerModel->refresh( myModel, myIds );
 
     ///
     ///  Peers tab
@@ -850,9 +798,9 @@ Details :: refresh( )
             if( !codeTip.isEmpty() )
                 codeTip.resize( codeTip.size()-1 ); // eat the trailing linefeed
 
-            item->setText( COL_UP, peer.rateToPeer.isZero() ? "" : Utils::speedToString( peer.rateToPeer ) );
-            item->setText( COL_DOWN, peer.rateToClient.isZero() ? "" : Utils::speedToString( peer.rateToClient ) );
-            item->setText( COL_PERCENT, peer.progress > 0 ? QString( "%1%" ).arg( locale.toString((int)(peer.progress*100.0))) : "" );
+            item->setText( COL_UP, peer.rateToPeer.isZero() ? "" : Formatter::speedToString( peer.rateToPeer ) );
+            item->setText( COL_DOWN, peer.rateToClient.isZero() ? "" : Formatter::speedToString( peer.rateToClient ) );
+            item->setText( COL_PERCENT, peer.progress > 0 ? QString( "%1%" ).arg( (int)( peer.progress * 100.0 ) ) : "" );
             item->setText( COL_STATUS, code );
             item->setToolTip( COL_STATUS, codeTip );
 
@@ -903,7 +851,6 @@ Details :: createInfoTab( )
     hig->addRow( tr( "Availability:" ), myAvailabilityLabel = new SqueezeLabel );
     hig->addRow( tr( "Downloaded:" ), myDownloadedLabel = new SqueezeLabel );
     hig->addRow( tr( "Uploaded:" ), myUploadedLabel = new SqueezeLabel );
-    hig->addRow( tr( "Ratio:" ), myRatioLabel = new SqueezeLabel );
     hig->addRow( tr( "State:" ), myStateLabel = new SqueezeLabel );
     hig->addRow( tr( "Running time:" ), myRunTimeLabel = new SqueezeLabel );
     hig->addRow( tr( "Remaining time:" ), myETALabel = new SqueezeLabel );
@@ -931,72 +878,83 @@ Details :: createInfoTab( )
 ****
 ***/
 
-void
-Details :: onShowBackupTrackersToggled( bool val )
-{
-    myPrefs.set( Prefs::SHOW_BACKUP_TRACKERS, val );
-}
-
 void
 Details :: onShowTrackerScrapesToggled( bool val )
 {
     myPrefs.set( Prefs::SHOW_TRACKER_SCRAPES, val );
 }
 
+void
+Details :: onShowBackupTrackersToggled( bool val )
+{
+    myPrefs.set( Prefs::SHOW_BACKUP_TRACKERS, val );
+}
+
 void
 Details :: onHonorsSessionLimitsToggled( bool val )
 {
     mySession.torrentSet( myIds, "honorsSessionLimits", val );
+    getNewData( );
 }
 void
 Details :: onDownloadLimitedToggled( bool val )
 {
     mySession.torrentSet( myIds, "downloadLimited", val );
+    getNewData( );
 }
 void
 Details :: onDownloadLimitChanged( int val )
 {
     mySession.torrentSet( myIds, "downloadLimit", val );
+    getNewData( );
 }
 void
 Details :: onUploadLimitedToggled( bool val )
 {
     mySession.torrentSet( myIds, "uploadLimited", val );
+    getNewData( );
 }
 void
 Details :: onUploadLimitChanged( int val )
 {
     mySession.torrentSet( myIds, "uploadLimit", val );
-}
-
-#define RATIO_KEY "seedRatioMode"
-
-void
-Details :: onSeedUntilChanged( bool b )
-{
-    if( b )
-        mySession.torrentSet( myIds, RATIO_KEY, sender()->property(RATIO_KEY).toInt() );
+    getNewData( );
 }
 
 void
-Details :: onSeedRatioLimitChanged( double val )
+Details :: onIdleModeChanged( int index )
 {
-    QSet<int> ids;
+    const int val = myIdleCombo->itemData( index ).toInt( );
+    mySession.torrentSet( myIds, "seedIdleMode", val );
+    getNewData( );
+}
 
-    foreach( int id, myIds ) {
-        const Torrent * tor = myModel.getTorrentFromId( id );
-        if( tor && tor->seedRatioLimit( ) )
-            ids.insert( id );
-    }
+void
+Details :: onIdleLimitChanged( int val )
+{
+    mySession.torrentSet( myIds, "seedIdleLimit", val );
+    getNewData( );
+}
 
-    if( !ids.empty( ) )
-        mySession.torrentSet( ids, "seedRatioLimit", val );
+void
+Details :: onRatioModeChanged( int index )
+{
+    const int val = myRatioCombo->itemData( index ).toInt( );
+    mySession.torrentSet( myIds, "seedRatioMode", val );
+}
+
+void
+Details :: onRatioLimitChanged( double val )
+{
+    mySession.torrentSet( myIds, "seedRatioLimit", val );
+    getNewData( );
 }
 
 void
 Details :: onMaxPeersChanged( int val )
 {
     mySession.torrentSet( myIds, "peer-limit", val );
+    getNewData( );
 }
 
 void
@@ -1006,19 +964,126 @@ Details :: onBandwidthPriorityChanged( int index )
     {
         const int priority = myBandwidthPriorityCombo->itemData(index).toInt( );
         mySession.torrentSet( myIds, "bandwidthPriority", priority );
+        getNewData( );
     }
 }
 
+void
+Details :: onTrackerSelectionChanged( )
+{
+    const int selectionCount = myTrackerView->selectionModel()->selectedRows().size();
+    myEditTrackerButton->setEnabled( selectionCount == 1 );
+    myRemoveTrackerButton->setEnabled( selectionCount > 0 );
+}
+
+void
+Details :: onAddTrackerClicked( )
+{
+    bool ok = false;
+    const QString url = QInputDialog::getText( this,
+                                               tr( "Add URL " ),
+                                               tr( "Add tracker announce URL:" ),
+                                               QLineEdit::Normal, QString(), &ok );
+    if( !ok )
+    {
+        // user pressed "cancel" -- noop
+    }
+    else if( !QUrl(url).isValid( ) )
+    {
+        QMessageBox::warning( this, tr( "Error" ), tr( "Invalid URL \"%1\"" ).arg( url ) );
+    }
+    else
+    {
+        QSet<int> ids;
+
+        foreach( int id, myIds )
+            if( myTrackerModel->find( id, url ) == -1 )
+                ids.insert( id );
+
+        if( ids.empty( ) ) // all the torrents already have this tracker
+        {
+            QMessageBox::warning( this, tr( "Error" ), tr( "Tracker already exists." ) );
+        }
+        else
+        {
+            QStringList urls;
+            urls << url;
+            mySession.torrentSet( ids, "trackerAdd", urls );
+            getNewData( );
+        }
+    }
+}
+
+void
+Details :: onEditTrackerClicked( )
+{
+    QItemSelectionModel * selectionModel = myTrackerView->selectionModel( );
+    QModelIndexList selectedRows = selectionModel->selectedRows( );
+    assert( selectedRows.size( ) == 1 );
+    QModelIndex i = selectionModel->currentIndex( );
+    const TrackerInfo trackerInfo = myTrackerView->model()->data( i, TrackerModel::TrackerRole ).value<TrackerInfo>();
+
+    bool ok = false;
+    const QString newval = QInputDialog::getText( this,
+                                                  tr( "Edit URL " ),
+                                                  tr( "Edit tracker announce URL:" ),
+                                                  QLineEdit::Normal,
+                                                  trackerInfo.st.announce, &ok );
+
+    if( !ok )
+    {
+        // user pressed "cancel" -- noop
+    }
+    else if( !QUrl(newval).isValid( ) )
+    {
+        QMessageBox::warning( this, tr( "Error" ), tr( "Invalid URL \"%1\"" ).arg( newval ) );
+    }
+    else
+    {
+        QSet<int> ids;
+        ids << trackerInfo.torrentId;
+
+        const QPair<int,QString> idUrl = qMakePair( trackerInfo.st.id, newval );
+
+        mySession.torrentSet( ids, "trackerReplace", idUrl );
+        getNewData( );
+    }
+}
+
+void
+Details :: onRemoveTrackerClicked( )
+{
+    // make a map of torrentIds to announce URLs to remove
+    QItemSelectionModel * selectionModel = myTrackerView->selectionModel( );
+    QModelIndexList selectedRows = selectionModel->selectedRows( );
+    QMap<int,int> torrentId_to_trackerIds;
+    foreach( QModelIndex i, selectedRows )
+    {
+        const TrackerInfo inf = myTrackerView->model()->data( i, TrackerModel::TrackerRole ).value<TrackerInfo>();
+        torrentId_to_trackerIds.insertMulti( inf.torrentId, inf.st.id );
+    }
+
+    // batch all of a tracker's torrents into one command
+    foreach( int id, torrentId_to_trackerIds.uniqueKeys( ) )
+    {
+        QSet<int> ids;
+        ids << id;
+        mySession.torrentSet( ids, "trackerRemove", torrentId_to_trackerIds.values( id ) );
+    }
+
+    selectionModel->clearSelection( );
+    getNewData( );
+}
+
 QWidget *
 Details :: createOptionsTab( )
 {
-    //QWidget * l;
     QSpinBox * s;
     QCheckBox * c;
     QComboBox * m;
     QHBoxLayout * h;
-    QRadioButton * r;
     QDoubleSpinBox * ds;
+    const QString speed_K_str = Formatter::unitStr( Formatter::SPEED, Formatter::KB );
 
     HIG * hig = new HIG( this );
     hig->addSectionTitle( tr( "Speed" ) );
@@ -1028,21 +1093,23 @@ Details :: createOptionsTab( )
     hig->addWideControl( c );
     connect( c, SIGNAL(clicked(bool)), this, SLOT(onHonorsSessionLimitsToggled(bool)) );
 
-    c = new QCheckBox( tr( "Limit &download speed (KiB/s):" ) );
+    c = new QCheckBox( tr( "Limit &download speed (%1):" ).arg( speed_K_str ) );
     mySingleDownCheck = c;
     s = new QSpinBox( );
-    mySingleDownSpin = s;
+    s->setSingleStep( 5 );
     s->setRange( 0, INT_MAX );
+    mySingleDownSpin = s;
     hig->addRow( c, s );
     enableWhenChecked( c, s );
     connect( c, SIGNAL(clicked(bool)), this, SLOT(onDownloadLimitedToggled(bool)) );
     connect( s, SIGNAL(valueChanged(int)), this, SLOT(onDownloadLimitChanged(int)));
 
-    c = new QCheckBox( tr( "Limit &upload speed (KiB/s):" ) );
+    c = new QCheckBox( tr( "Limit &upload speed (%1):" ).arg( speed_K_str ) );
     mySingleUpCheck = c;
     s = new QSpinBox( );
-    mySingleUpSpin = s;
+    s->setSingleStep( 5 );
     s->setRange( 0, INT_MAX );
+    mySingleUpSpin = s;
     hig->addRow( c, s );
     enableWhenChecked( c, s );
     connect( c, SIGNAL(clicked(bool)), this, SLOT(onUploadLimitedToggled(bool)) );
@@ -1057,38 +1124,43 @@ Details :: createOptionsTab( )
     myBandwidthPriorityCombo = m;
 
     hig->addSectionDivider( );
-    hig->addSectionTitle( tr( "Seed-Until Ratio" ) );
-
-    r = new QRadioButton( tr( "Use &global settings" ) );
-    r->setProperty( RATIO_KEY, TR_RATIOLIMIT_GLOBAL );
-    connect( r, SIGNAL(clicked(bool)), this, SLOT(onSeedUntilChanged(bool)));
-    mySeedGlobalRadio = r;
-    hig->addWideControl( r );
-
-    r = new QRadioButton( tr( "Seed &regardless of ratio" ) );
-    r->setProperty( RATIO_KEY, TR_RATIOLIMIT_UNLIMITED );
-    connect( r, SIGNAL(clicked(bool)), this, SLOT(onSeedUntilChanged(bool)));
-    mySeedForeverRadio = r;
-    hig->addWideControl( r );
+    hig->addSectionTitle( tr( "Seeding Limits" ) );
 
     h = new QHBoxLayout( );
     h->setSpacing( HIG :: PAD );
-    r = new QRadioButton( tr( "&Seed torrent until its ratio reaches:" ) );
-    r->setProperty( RATIO_KEY, TR_RATIOLIMIT_SINGLE );
-    connect( r, SIGNAL(clicked(bool)), this, SLOT(onSeedUntilChanged(bool)));
-    mySeedCustomRadio = r;
-    h->addWidget( r );
+    m = new QComboBox;
+    m->addItem( tr( "Use Global Settings" ),      TR_RATIOLIMIT_GLOBAL );
+    m->addItem( tr( "Seed regardless of ratio" ), TR_RATIOLIMIT_UNLIMITED );
+    m->addItem( tr( "Stop seeding at ratio:" ),   TR_RATIOLIMIT_SINGLE );
+    connect( m, SIGNAL(currentIndexChanged(int)), this, SLOT(onRatioModeChanged(int)));
+    h->addWidget( myRatioCombo = m );
     ds = new QDoubleSpinBox( );
     ds->setRange( 0.5, INT_MAX );
-    connect( ds, SIGNAL(valueChanged(double)), this, SLOT(onSeedRatioLimitChanged(double)));
-    mySeedCustomSpin = ds;
-    h->addWidget( ds );
-    hig->addWideControl( h );
+    connect( ds, SIGNAL(valueChanged(double)), this, SLOT(onRatioLimitChanged(double)));
+    h->addWidget( myRatioSpin = ds );
+    hig->addRow( tr( "&Ratio:" ), h, m );
+
+    h = new QHBoxLayout( );
+    h->setSpacing( HIG :: PAD );
+    m = new QComboBox;
+    m->addItem( tr( "Use Global Settings" ),                 TR_IDLELIMIT_GLOBAL );
+    m->addItem( tr( "Seed regardless of activity" ),         TR_IDLELIMIT_UNLIMITED );
+    m->addItem( tr( "Stop seeding if idle for N minutes:" ), TR_IDLELIMIT_SINGLE );
+    connect( m, SIGNAL(currentIndexChanged(int)), this, SLOT(onIdleModeChanged(int)));
+    h->addWidget( myIdleCombo = m );
+    s = new QSpinBox( );
+    s->setSingleStep( 5 );
+    s->setRange( 1, 9999 );
+    connect( s, SIGNAL(valueChanged(int)), this, SLOT(onIdleLimitChanged(int)));
+    h->addWidget( myIdleSpin = s );
+    hig->addRow( tr( "&Idle:" ), h, m );
+
 
     hig->addSectionDivider( );
     hig->addSectionTitle( tr( "Peer Connections" ) );
 
     s = new QSpinBox( );
+    s->setSingleStep( 5 );
     s->setRange( 1, 300 );
     connect( s, SIGNAL(valueChanged(int)), this, SLOT(onMaxPeersChanged(int)));
     myPeerLimitSpin = s;
@@ -1107,21 +1179,65 @@ QWidget *
 Details :: createTrackerTab( )
 {
     QCheckBox * c;
+    QPushButton * p;
     QWidget * top = new QWidget;
     QVBoxLayout * v = new QVBoxLayout( top );
+    QHBoxLayout * h = new QHBoxLayout();
+    QVBoxLayout * v2 = new QVBoxLayout();
 
-    v->setSpacing( HIG :: PAD_BIG );
+    v->setSpacing( HIG::PAD_BIG );
     v->setContentsMargins( HIG::PAD_BIG, HIG::PAD_BIG, HIG::PAD_BIG, HIG::PAD_BIG );
 
-    QStringList headers;
-    headers << tr("Trackers");
-    myTrackerTree = new QTreeWidget;
-    myTrackerTree->setHeaderLabels( headers );
-    myTrackerTree->setSelectionMode( QTreeWidget::NoSelection );
-    myTrackerTree->setRootIsDecorated( false );
-    myTrackerTree->setTextElideMode( Qt::ElideRight );
-    myTrackerTree->setAlternatingRowColors( true );
-    v->addWidget( myTrackerTree, 1 );
+    h->setSpacing( HIG::PAD );
+    h->setContentsMargins( HIG::PAD_SMALL, HIG::PAD_SMALL, HIG::PAD_SMALL, HIG::PAD_SMALL );
+
+    v2->setSpacing( HIG::PAD );
+
+    myTrackerModel = new TrackerModel;
+    myTrackerFilter = new TrackerModelFilter;
+    myTrackerFilter->setSourceModel( myTrackerModel );
+    myTrackerView = new QTreeView;
+    myTrackerView->setModel( myTrackerFilter );
+    myTrackerView->setHeaderHidden( true );
+    myTrackerView->setSelectionMode( QTreeWidget::ExtendedSelection );
+    myTrackerView->setRootIsDecorated( false );
+    myTrackerView->setIndentation( 2 );
+    myTrackerView->setItemsExpandable( false );
+    myTrackerView->setAlternatingRowColors( true );
+    myTrackerView->setItemDelegate( myTrackerDelegate = new TrackerDelegate( ) );
+    connect( myTrackerView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(onTrackerSelectionChanged()));
+    h->addWidget( myTrackerView, 1 );
+
+    p = new QPushButton();
+    p->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
+    p->setToolTip( "Add Tracker" );
+    myAddTrackerButton = p;
+    v2->addWidget( p, 1 );
+    connect( p, SIGNAL(clicked(bool)), this, SLOT(onAddTrackerClicked()));
+
+    p = new QPushButton();
+    p->setIcon( getStockIcon( "document-properties", QStyle::SP_DesktopIcon ) );
+    p->setToolTip( "Edit Tracker" );
+    myAddTrackerButton = p;
+    p->setEnabled( false );
+    myEditTrackerButton = p;
+    v2->addWidget( p, 1 );
+    connect( p, SIGNAL(clicked(bool)), this, SLOT(onEditTrackerClicked()));
+
+    p = new QPushButton();
+    p->setIcon( getStockIcon( "list-remove", QStyle::SP_TrashIcon ) );
+    p->setToolTip( "Remove Trackers" );
+    p->setEnabled( false );
+    myRemoveTrackerButton = p;
+    v2->addWidget( p, 1 );
+    connect( p, SIGNAL(clicked(bool)), this, SLOT(onRemoveTrackerClicked()));
+
+    v2->addStretch( 1 );
+
+    h->addLayout( v2, 1 );
+    h->setStretch( 1, 0 );
+
+    v->addLayout( h, 1 );
 
     c = new QCheckBox( tr( "Show &more details" ) );
     c->setChecked( myPrefs.getBool( Prefs::SHOW_TRACKER_SCRAPES ) );
@@ -1157,6 +1273,7 @@ Details :: createPeersTab( )
     myPeerTree->setHeaderLabels( headers );
     myPeerTree->setColumnWidth( 0, 20 );
     myPeerTree->setSortingEnabled( true );
+    myPeerTree->sortByColumn( COL_ADDRESS, Qt::AscendingOrder );
     myPeerTree->setRootIsDecorated( false );
     myPeerTree->setTextElideMode( Qt::ElideRight );
     v->addWidget( myPeerTree, 1 );
@@ -1206,6 +1323,7 @@ Details :: onFilePriorityChanged( const QSet<int>& indices, int priority )
         default:           key = "priority-normal"; break;
     }
     mySession.torrentSet( myIds, key, indices.toList( ) );
+    getNewData( );
 }
 
 void
@@ -1213,4 +1331,5 @@ Details :: onFileWantedChanged( const QSet<int>& indices, bool wanted )
 {
     QString key( wanted ? "files-wanted" : "files-unwanted" );
     mySession.torrentSet( myIds, key, indices.toList( ) );
+    getNewData( );
 }
