@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: details.h 10627 2010-05-05 09:41:14Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: details.h 11234 2010-09-19 17:33:25Z charles $
  */
 
 #ifndef DETAILS_DIALOG_H
@@ -36,11 +36,17 @@ class QTreeWidgetItem;
 class Session;
 class Torrent;
 class TorrentModel;
+class TrackerDelegate;
+class TrackerModel;
+class TrackerModelFilter;
 
 class Details: public QDialog
 {
         Q_OBJECT
 
+    private:
+        void getNewData( );
+
     private slots:
         void onTorrentChanged( );
         void onTimer( );
@@ -58,12 +64,12 @@ class Details: public QDialog
         QWidget * createOptionsTab( );
 
     private:
+        QIcon getStockIcon( const QString& freedesktop_name, int fallback );
         QString timeToStringRounded( int seconds );
         QString trimToDesiredWidth( const QString& str );
         void enableWhenChecked( QCheckBox *, QWidget * );
 
     private:
-
         Session& mySession;
         Prefs& myPrefs;
         TorrentModel& myModel;
@@ -77,7 +83,6 @@ class Details: public QDialog
         QLabel * myAvailabilityLabel;
         QLabel * myDownloadedLabel;
         QLabel * myUploadedLabel;
-        QLabel * myRatioLabel;
         QLabel * myErrorLabel;
         QLabel * myRunTimeLabel;
         QLabel * myETALabel;
@@ -88,12 +93,15 @@ class Details: public QDialog
         QCheckBox * mySingleUpCheck;
         QCheckBox * myShowTrackerScrapesCheck;
         QCheckBox * myShowBackupTrackersCheck;
+        QPushButton * myAddTrackerButton;
+        QPushButton * myEditTrackerButton;
+        QPushButton * myRemoveTrackerButton;
         QSpinBox * mySingleDownSpin;
         QSpinBox * mySingleUpSpin;
-        QRadioButton * mySeedGlobalRadio;
-        QRadioButton * mySeedForeverRadio;
-        QRadioButton * mySeedCustomRadio;
-        QDoubleSpinBox * mySeedCustomSpin;
+        QComboBox * myRatioCombo;
+        QDoubleSpinBox * myRatioSpin;
+        QComboBox * myIdleCombo;
+        QSpinBox * myIdleSpin;
         QSpinBox * myPeerLimitSpin;
         QComboBox * myBandwidthPriorityCombo;
 
@@ -113,15 +121,22 @@ class Details: public QDialog
         QLabel * myAnnounceResponseLabel;
         QLabel * myAnnounceManualLabel;
 
-        QTreeWidget * myTrackerTree;
+        TrackerModel * myTrackerModel;
+        TrackerModelFilter * myTrackerFilter;
+        TrackerDelegate * myTrackerDelegate;
+        QTreeView * myTrackerView;
+        //QMap<QString,QTreeWidgetItem*> myTrackerTiers;
+        //QMap<QString,QTreeWidgetItem*> myTrackerItems;
+
         QTreeWidget * myPeerTree;
-        QMap<QString,QTreeWidgetItem*> myTrackerStats;
         QMap<QString,QTreeWidgetItem*> myPeers;
+
         QWidgetList myWidgets;
 
         FileTreeView * myFileTreeView;
 
     private slots:
+        void refreshPref( int key );
         void onBandwidthPriorityChanged( int );
         void onFilePriorityChanged( const QSet<int>& fileIndices, int );
         void onFileWantedChanged( const QSet<int>& fileIndices, bool );
@@ -130,10 +145,16 @@ class Details: public QDialog
         void onDownloadLimitChanged( int );
         void onUploadLimitedToggled( bool );
         void onUploadLimitChanged( int );
-        void onSeedUntilChanged( bool );
-        void onSeedRatioLimitChanged( double );
-        void onShowBackupTrackersToggled( bool );
+        void onRatioModeChanged( int );
+        void onRatioLimitChanged( double );
+        void onIdleModeChanged( int );
+        void onIdleLimitChanged( int );
         void onShowTrackerScrapesToggled( bool );
+        void onShowBackupTrackersToggled( bool );
+        void onTrackerSelectionChanged( );
+        void onAddTrackerClicked( );
+        void onEditTrackerClicked( );
+        void onRemoveTrackerClicked( );
         void onMaxPeersChanged( int );
         void refresh( );
 };
