@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: prefs.cc 10638 2010-05-08 22:42:28Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: prefs.cc 11379 2010-11-02 01:24:08Z charles $
  */
 
 #include <cassert>
@@ -42,11 +42,11 @@ Prefs::PrefItem Prefs::myItems[] =
     { SORT_MODE, "sort-mode", TrTypes::SortModeType },
     { SORT_REVERSED, "sort-reversed", QVariant::Bool },
     { COMPACT_VIEW, "compact-view", QVariant::Bool },
-    { SHOW_BACKUP_TRACKERS, "show-backup-trackers", QVariant::Bool },
     { FILTERBAR, "show-filterbar", QVariant::Bool },
     { STATUSBAR, "show-statusbar", QVariant::Bool },
     { STATUSBAR_STATS, "statusbar-stats", QVariant::String },
-    { SHOW_TRACKER_SCRAPES, "show-tracker-scrapes", QVariant::Bool },
+    { SHOW_TRACKER_SCRAPES, "show-extra-peer-details", QVariant::Bool },
+    { SHOW_BACKUP_TRACKERS, "show-backup-trackers", QVariant::Bool },
     { TOOLBAR, "show-toolbar" , QVariant::Bool },
     { BLOCKLIST_DATE, "blocklist-date", QVariant::DateTime },
     { BLOCKLIST_UPDATES_ENABLED, "blocklist-updates-enabled" , QVariant::Bool },
@@ -56,6 +56,8 @@ Prefs::PrefItem Prefs::myItems[] =
     { MAIN_WINDOW_X, "main-window-x", QVariant::Int },
     { MAIN_WINDOW_Y, "main-window-y", QVariant::Int },
     { FILTER_MODE, "filter-mode", TrTypes::FilterModeType },
+    { FILTER_TRACKERS, "filter-trackers", QVariant::String },
+    { FILTER_TEXT, "filter-text", QVariant::String },
     { SESSION_IS_REMOTE, "remote-session-enabled", QVariant::Bool },
     { SESSION_REMOTE_HOST, "remote-session-host", QVariant::String },
     { SESSION_REMOTE_PORT, "remote-session-port", QVariant::Int },
@@ -65,18 +67,21 @@ Prefs::PrefItem Prefs::myItems[] =
     { USER_HAS_GIVEN_INFORMED_CONSENT, "user-has-given-informed-consent", QVariant::Bool },
 
     /* libtransmission settings */
-    { ALT_SPEED_LIMIT_UP, TR_PREFS_KEY_ALT_SPEED_UP, QVariant::Int },
-    { ALT_SPEED_LIMIT_DOWN, TR_PREFS_KEY_ALT_SPEED_DOWN, QVariant::Int },
+    { ALT_SPEED_LIMIT_UP, TR_PREFS_KEY_ALT_SPEED_UP_KBps, QVariant::Int },
+    { ALT_SPEED_LIMIT_DOWN, TR_PREFS_KEY_ALT_SPEED_DOWN_KBps, QVariant::Int },
     { ALT_SPEED_LIMIT_ENABLED, TR_PREFS_KEY_ALT_SPEED_ENABLED, QVariant::Bool },
     { ALT_SPEED_LIMIT_TIME_BEGIN, TR_PREFS_KEY_ALT_SPEED_TIME_BEGIN, QVariant::Int },
     { ALT_SPEED_LIMIT_TIME_END, TR_PREFS_KEY_ALT_SPEED_TIME_END, QVariant::Int },
     { ALT_SPEED_LIMIT_TIME_ENABLED, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED, QVariant::Bool },
     { ALT_SPEED_LIMIT_TIME_DAY, TR_PREFS_KEY_ALT_SPEED_TIME_DAY, QVariant::Int },
     { BLOCKLIST_ENABLED, TR_PREFS_KEY_BLOCKLIST_ENABLED, QVariant::Bool },
-    { DSPEED, TR_PREFS_KEY_DSPEED, QVariant::Int },
+    { BLOCKLIST_URL, TR_PREFS_KEY_BLOCKLIST_URL, QVariant::String },
+    { DSPEED, TR_PREFS_KEY_DSPEED_KBps, QVariant::Int },
     { DSPEED_ENABLED, TR_PREFS_KEY_DSPEED_ENABLED, QVariant::Bool },
     { DOWNLOAD_DIR, TR_PREFS_KEY_DOWNLOAD_DIR, QVariant::String },
     { ENCRYPTION, TR_PREFS_KEY_ENCRYPTION, QVariant::Int },
+    { IDLE_LIMIT, TR_PREFS_KEY_IDLE_LIMIT, QVariant::Int },
+    { IDLE_LIMIT_ENABLED, TR_PREFS_KEY_IDLE_LIMIT_ENABLED, QVariant::Bool },
     { INCOMPLETE_DIR, TR_PREFS_KEY_INCOMPLETE_DIR, QVariant::String },
     { INCOMPLETE_DIR_ENABLED, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED, QVariant::Bool },
     { LAZY_BITFIELD, TR_PREFS_KEY_LAZY_BITFIELD, QVariant::Bool },
@@ -97,14 +102,7 @@ Prefs::PrefItem Prefs::myItems[] =
     { DHT_ENABLED, TR_PREFS_KEY_DHT_ENABLED, QVariant::Bool },
     { LPD_ENABLED, TR_PREFS_KEY_LPD_ENABLED, QVariant::Bool },
     { PORT_FORWARDING, TR_PREFS_KEY_PORT_FORWARDING, QVariant::Bool },
-    { PROXY_AUTH_ENABLED, TR_PREFS_KEY_PROXY_AUTH_ENABLED, QVariant::Bool },
     { PREALLOCATION, TR_PREFS_KEY_PREALLOCATION, QVariant::Int },
-    { PROXY_ENABLED, TR_PREFS_KEY_PROXY_ENABLED, QVariant::Bool },
-    { PROXY_PASSWORD, TR_PREFS_KEY_PROXY_PASSWORD, QVariant::String },
-    { PROXY_PORT, TR_PREFS_KEY_PROXY_PORT, QVariant::Int },
-    { PROXY, TR_PREFS_KEY_PROXY, QVariant::String },
-    { PROXY_TYPE, TR_PREFS_KEY_PROXY_TYPE, QVariant::Int },
-    { PROXY_USERNAME, TR_PREFS_KEY_PROXY_USERNAME, QVariant::String },
     { RATIO, TR_PREFS_KEY_RATIO, QVariant::Double },
     { RATIO_ENABLED, TR_PREFS_KEY_RATIO_ENABLED, QVariant::Bool },
     { RENAME_PARTIAL_FILES, TR_PREFS_KEY_RENAME_PARTIAL_FILES, QVariant::Bool },
@@ -116,7 +114,7 @@ Prefs::PrefItem Prefs::myItems[] =
     { RPC_WHITELIST_ENABLED, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, QVariant::Bool },
     { RPC_WHITELIST, TR_PREFS_KEY_RPC_WHITELIST, QVariant::String },
     { USPEED_ENABLED, TR_PREFS_KEY_USPEED_ENABLED, QVariant::Bool },
-    { USPEED, TR_PREFS_KEY_USPEED, QVariant::Int },
+    { USPEED, TR_PREFS_KEY_USPEED_KBps, QVariant::Int },
     { UPLOAD_SLOTS_PER_TORRENT, TR_PREFS_KEY_UPLOAD_SLOTS_PER_TORRENT, QVariant::Int }
 };
 
@@ -131,6 +129,10 @@ Prefs :: Prefs( const char * configDir ):
     for( int i=0; i<PREFS_COUNT; ++i )
         assert( myItems[i].id == i );
 
+    // these are the prefs that don't get saved to settings.json
+    // when the application exits.
+    myTemporaryPrefs << FILTER_TEXT;
+
     tr_benc top;
     tr_bencInitDict( &top, 0 );
     initDefaults( &top );
@@ -195,10 +197,16 @@ Prefs :: ~Prefs( )
         tr_bencInitDict( &top, PREFS_COUNT );
 
     /* merge our own settings with the ones already in the file */
-    for( int i=0; i<PREFS_COUNT; ++i ) {
+    for( int i=0; i<PREFS_COUNT; ++i )
+    {
+        if( myTemporaryPrefs.contains( i ) )
+            continue;
+
         const char * key = myItems[i].key;
         const QVariant& val = myValues[i];
-        switch( myItems[i].type ) {
+
+        switch( myItems[i].type )
+        {
             case QVariant::Int:
                 tr_bencDictAddInt( &top, key, val.toInt() );
                 break;
@@ -238,34 +246,35 @@ Prefs :: ~Prefs( )
 void
 Prefs :: initDefaults( tr_benc * d )
 {
-    tr_bencDictAddStr( d, keyStr(DIR_WATCH), tr_getDefaultDownloadDir( ) );
-    tr_bencDictAddInt( d, keyStr(DIR_WATCH_ENABLED), false );
-    tr_bencDictAddInt( d, keyStr(INHIBIT_HIBERNATION), false );
-    tr_bencDictAddInt( d, keyStr(BLOCKLIST_DATE), 0 );
-    tr_bencDictAddInt( d, keyStr(BLOCKLIST_UPDATES_ENABLED), true );
-    tr_bencDictAddStr( d, keyStr(OPEN_DIALOG_FOLDER), QDir::home().absolutePath().toLatin1() );
-    tr_bencDictAddInt( d, keyStr(SHOW_BACKUP_TRACKERS), false );
-    tr_bencDictAddInt( d, keyStr(SHOW_TRACKER_SCRAPES), false );
-    tr_bencDictAddInt( d, keyStr(TOOLBAR), true );
-    tr_bencDictAddInt( d, keyStr(FILTERBAR), true );
-    tr_bencDictAddInt( d, keyStr(STATUSBAR), true );
-    tr_bencDictAddInt( d, keyStr(SHOW_TRAY_ICON), false );
-    tr_bencDictAddInt( d, keyStr(SHOW_DESKTOP_NOTIFICATION), true );
-    tr_bencDictAddStr( d, keyStr(STATUSBAR_STATS), "total-ratio" );
-    tr_bencDictAddInt( d, keyStr(OPTIONS_PROMPT), true );
-    tr_bencDictAddInt( d, keyStr(MAIN_WINDOW_HEIGHT), 500 );
-    tr_bencDictAddInt( d, keyStr(MAIN_WINDOW_WIDTH), 300 );
-    tr_bencDictAddInt( d, keyStr(MAIN_WINDOW_X), 50 );
-    tr_bencDictAddInt( d, keyStr(MAIN_WINDOW_Y), 50 );
-    tr_bencDictAddStr( d, keyStr(FILTER_MODE), "all" );
-    tr_bencDictAddStr( d, keyStr(MAIN_WINDOW_LAYOUT_ORDER), "menu,toolbar,filter,list,statusbar" );
-    tr_bencDictAddStr( d, keyStr(DOWNLOAD_DIR), tr_getDefaultDownloadDir( ) );
-    tr_bencDictAddInt( d, keyStr(ASKQUIT), true );
-    tr_bencDictAddStr( d, keyStr(SORT_MODE), "sort-by-name" );
-    tr_bencDictAddInt( d, keyStr(SORT_REVERSED), false );
-    tr_bencDictAddInt( d, keyStr(COMPACT_VIEW), false );
-    tr_bencDictAddStr( d, keyStr(SESSION_REMOTE_HOST), "localhost" );
-    tr_bencDictAddInt( d, keyStr(SESSION_REMOTE_PORT), atoi(TR_DEFAULT_RPC_PORT_STR) );
+    tr_bencDictAddStr ( d, keyStr(DIR_WATCH), tr_getDefaultDownloadDir( ) );
+    tr_bencDictAddBool( d, keyStr(DIR_WATCH_ENABLED), false );
+    tr_bencDictAddBool( d, keyStr(INHIBIT_HIBERNATION), false );
+    tr_bencDictAddInt ( d, keyStr(BLOCKLIST_DATE), 0 );
+    tr_bencDictAddBool( d, keyStr(BLOCKLIST_UPDATES_ENABLED), true );
+    tr_bencDictAddStr ( d, keyStr(OPEN_DIALOG_FOLDER), QDir::home().absolutePath().toLatin1() );
+    tr_bencDictAddInt ( d, keyStr(SHOW_TRACKER_SCRAPES), false );
+    tr_bencDictAddBool( d, keyStr(TOOLBAR), true );
+    tr_bencDictAddBool( d, keyStr(FILTERBAR), true );
+    tr_bencDictAddBool( d, keyStr(STATUSBAR), true );
+    tr_bencDictAddBool( d, keyStr(SHOW_TRAY_ICON), false );
+    tr_bencDictAddBool( d, keyStr(SHOW_DESKTOP_NOTIFICATION), true );
+    tr_bencDictAddStr ( d, keyStr(STATUSBAR_STATS), "total-ratio" );
+    tr_bencDictAddBool( d, keyStr(SHOW_TRACKER_SCRAPES), false );
+    tr_bencDictAddBool( d, keyStr(SHOW_BACKUP_TRACKERS), false );
+    tr_bencDictAddBool( d, keyStr(OPTIONS_PROMPT), true );
+    tr_bencDictAddInt ( d, keyStr(MAIN_WINDOW_HEIGHT), 500 );
+    tr_bencDictAddInt ( d, keyStr(MAIN_WINDOW_WIDTH), 300 );
+    tr_bencDictAddInt ( d, keyStr(MAIN_WINDOW_X), 50 );
+    tr_bencDictAddInt ( d, keyStr(MAIN_WINDOW_Y), 50 );
+    tr_bencDictAddStr ( d, keyStr(FILTER_MODE), "all" );
+    tr_bencDictAddStr ( d, keyStr(MAIN_WINDOW_LAYOUT_ORDER), "menu,toolbar,filter,list,statusbar" );
+    tr_bencDictAddStr ( d, keyStr(DOWNLOAD_DIR), tr_getDefaultDownloadDir( ) );
+    tr_bencDictAddBool( d, keyStr(ASKQUIT), true );
+    tr_bencDictAddStr ( d, keyStr(SORT_MODE), "sort-by-name" );
+    tr_bencDictAddBool( d, keyStr(SORT_REVERSED), false );
+    tr_bencDictAddBool( d, keyStr(COMPACT_VIEW), false );
+    tr_bencDictAddStr ( d, keyStr(SESSION_REMOTE_HOST), "localhost" );
+    tr_bencDictAddInt ( d, keyStr(SESSION_REMOTE_PORT), atoi(TR_DEFAULT_RPC_PORT_STR) );
     tr_bencDictAddBool( d, keyStr(SESSION_IS_REMOTE), false );
     tr_bencDictAddBool( d, keyStr(SESSION_REMOTE_AUTH), false );
     tr_bencDictAddBool( d, keyStr(USER_HAS_GIVEN_INFORMED_CONSENT), false );
