@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
- * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license.
- * This exemption does not extend to derived works not owned by
- * the Transmission project.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
- * $Id: prefs.h 10638 2010-05-08 22:42:28Z charles $
+ * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+ *
+ * $Id: prefs.h 11379 2010-11-02 01:24:08Z charles $
  */
 
 #ifndef QTR_PREFS_H
@@ -15,6 +15,7 @@
 
 #include <QDateTime>
 #include <QObject>
+#include <QSet>
 #include <QString>
 #include <QVariant>
 
@@ -45,11 +46,11 @@ class Prefs: public QObject
             SORT_MODE,
             SORT_REVERSED,
             COMPACT_VIEW,
-            SHOW_BACKUP_TRACKERS,
             FILTERBAR,
             STATUSBAR,
             STATUSBAR_STATS,
             SHOW_TRACKER_SCRAPES,
+            SHOW_BACKUP_TRACKERS,
             TOOLBAR,
             BLOCKLIST_DATE,
             BLOCKLIST_UPDATES_ENABLED,
@@ -59,6 +60,8 @@ class Prefs: public QObject
             MAIN_WINDOW_X,
             MAIN_WINDOW_Y,
             FILTER_MODE,
+            FILTER_TRACKERS,
+            FILTER_TEXT,
             SESSION_IS_REMOTE,
             SESSION_REMOTE_HOST,
             SESSION_REMOTE_PORT,
@@ -77,10 +80,13 @@ class Prefs: public QObject
             ALT_SPEED_LIMIT_TIME_ENABLED,
             ALT_SPEED_LIMIT_TIME_DAY,
             BLOCKLIST_ENABLED,
+            BLOCKLIST_URL,
             DSPEED,
             DSPEED_ENABLED,
             DOWNLOAD_DIR,
             ENCRYPTION,
+            IDLE_LIMIT,
+            IDLE_LIMIT_ENABLED,
             INCOMPLETE_DIR,
             INCOMPLETE_DIR_ENABLED,
             LAZY_BITFIELD,
@@ -101,14 +107,7 @@ class Prefs: public QObject
             DHT_ENABLED,
             LPD_ENABLED,
             PORT_FORWARDING,
-            PROXY_AUTH_ENABLED,
             PREALLOCATION,
-            PROXY_ENABLED,
-            PROXY_PASSWORD,
-            PROXY_PORT,
-            PROXY,
-            PROXY_TYPE,
-            PROXY_USERNAME,
             RATIO,
             RATIO_ENABLED,
             RENAME_PARTIAL_FILES,
@@ -138,6 +137,7 @@ class Prefs: public QObject
         static PrefItem myItems[];
 
     private:
+        QSet<int> myTemporaryPrefs;
         QString myConfigDir;
         QVariant myValues[PREFS_COUNT];
         void initDefaults( struct tr_benc* );
